@@ -61,9 +61,11 @@ pub struct Config {
     defines: Vec<(OsString, OsString)>,
     deps: Vec<String>,
     target: Option<String>,
+    host: Option<String>,
     out_dir: Option<PathBuf>,
     profile: Option<String>,
     build_args: Vec<OsString>,
+    cmake_target: Option<String>,
 }
 
 /// Builds the native library rooted at `path` with the default cmake options.
@@ -98,7 +100,9 @@ impl Config {
             profile: None,
             out_dir: None,
             target: None,
+            host: None,
             build_args: Vec::new(),
+            cmake_target: None,
         }
     }
 
@@ -137,6 +141,15 @@ impl Config {
         self
     }
 
+    /// Sets the host triple for this compilation.
+    ///
+    /// This is automatically scraped from `$HOST` which is set for Cargo
+    /// build scripts so it's not necessary to call this from a build script.
+    pub fn host(&mut self, host: &str) -> &mut Config {
+        self.host = Some(host.to_string());
+        self
+    }
+
     /// Sets the output directory for this compilation.
     ///
     /// This is automatically scraped from `$OUT_DIR` which is set for Cargo
@@ -161,6 +174,13 @@ impl Config {
         self
     }
 
+    /// Sets the build target for the final `cmake` build step, this will
+    /// default to "install" if not specified.
+    pub fn build_target(&mut self, target: &str) -> &mut Config {
+        self.cmake_target = Some(target.to_string());
+        self
+    }
+
     /// Run this configuration, compiling the library with all the configured
     /// options.
     ///
@@ -168,13 +188,28 @@ impl Config {
     /// command to build the library.
     pub fn build(&mut self) -> PathBuf {
         let target = self.target.clone().unwrap_or_else(|| {
-            env::var("TARGET").unwrap()
+            getenv_unwrap("TARGET")
+        });
+        let host = self.host.clone().unwrap_or_else(|| {
+            getenv_unwrap("HOST")
         });
         let msvc = target.contains("msvc");
-        let compiler = gcc::Config::new().get_compiler();
+        let c_compiler = gcc::Config::new().cargo_metadata(false)
+                                         .opt_level(0)
+                                         .debug(false)
+                                         .target(&target)
+                                         .host(&host)
+                                         .get_compiler();
+        let cxx_compiler = gcc::Config::new().cargo_metadata(false)
+                                         .cpp(true)
+                                         .opt_level(0)
+                                         .debug(false)
+                                         .target(&target)
+                                         .host(&host)
+                                         .get_compiler();
 
         let dst = self.out_dir.clone().unwrap_or_else(|| {
-            PathBuf::from(&env::var("OUT_DIR").unwrap())
+            PathBuf::from(getenv_unwrap("OUT_DIR"))
         });
         let build = dst.join("build");
         self.maybe_clear(&build);
@@ -208,7 +243,7 @@ impl Config {
             cmd.arg("-G").arg(self.visual_studio_generator(&target));
         }
         let profile = self.profile.clone().unwrap_or_else(|| {
-            match &env::var("PROFILE").unwrap()[..] {
+            match &getenv_unwrap("PROFILE")[..] {
                 "bench" | "release" => "Release",
                 // currently we need to always use the same CRT for MSVC
                 _ if msvc => "Release",
@@ -229,28 +264,44 @@ impl Config {
             cmd.arg(dstflag);
         }
 
-        if !self.defined("CMAKE_C_FLAGS") {
-            let mut cflagsflag = OsString::from("-DCMAKE_C_FLAGS=");
-            cflagsflag.push(&self.cflags);
-            for arg in compiler.args() {
-                cflagsflag.push(" ");
-                cflagsflag.push(arg);
-            }
-            cmd.arg(cflagsflag);
-        }
+        {
+            let mut set_compiler = |kind: &str,
+                                    compiler: &gcc::Tool,
+                                    extra: &OsString| {
+                let flag_var = format!("CMAKE_{}_FLAGS", kind);
+                let tool_var = format!("CMAKE_{}_COMPILER", kind);
+                if !self.defined(&flag_var) {
+                    let mut flagsflag = OsString::from("-D");
+                    flagsflag.push(&flag_var);
+                    flagsflag.push("=");
+                    flagsflag.push(extra);
+                    for arg in compiler.args() {
+                        flagsflag.push(" ");
+                        flagsflag.push(arg);
+                    }
+                    cmd.arg(flagsflag);
+                }
 
-        // Apparently cmake likes to have an absolute path to the compiler as
-        // otherwise it sometimes thinks that this variable changed as it thinks
-        // the found compiler, /usr/bin/cc, differs from the specified compiler,
-        // cc. Not entirely sure what's up, but at least this means cmake
-        // doesn't get confused?
-        //
-        // Also don't specify this on Windows as it's not needed for MSVC and
-        // for MinGW it doesn't really vary.
-        if !self.defined("CMAKE_C_COMPILER") && env::consts::FAMILY != "windows" {
-            let mut ccompiler = OsString::from("-DCMAKE_C_COMPILER=");
-            ccompiler.push(find_exe(compiler.path()));
-            cmd.arg(ccompiler);
+                // Apparently cmake likes to have an absolute path to the
+                // compiler as otherwise it sometimes thinks that this variable
+                // changed as it thinks the found compiler, /usr/bin/cc,
+                // differs from the specified compiler, cc. Not entirely sure
+                // what's up, but at least this means cmake doesn't get
+                // confused?
+                //
+                // Also don't specify this on Windows as it's not needed for
+                // MSVC and for MinGW it doesn't really vary.
+                if !self.defined(&tool_var) && env::consts::FAMILY != "windows" {
+                    let mut ccompiler = OsString::from("-D");
+                    ccompiler.push(&tool_var);
+                    ccompiler.push("=");
+                    ccompiler.push(find_exe(compiler.path()));
+                    cmd.arg(ccompiler);
+                }
+            };
+
+            set_compiler("C", &c_compiler, &self.cflags);
+            set_compiler("CXX", &cxx_compiler, &OsString::new());
         }
 
         if !self.defined("CMAKE_BUILD_TYPE") {
@@ -267,9 +318,10 @@ impl Config {
         }
 
         // And build!
+        let target = self.cmake_target.clone().unwrap_or("install".to_string());
         run(Command::new("cmake")
                     .arg("--build").arg(".")
-                    .arg("--target").arg("install")
+                    .arg("--target").arg(target)
                     .arg("--config").arg(profile)
                     .arg("--").args(&self.build_args)
                     .args(&parallel_args)
@@ -317,12 +369,16 @@ impl Config {
             Some(src) => src,
             None => return,
         };
-        let mut contents = String::new();
         let mut f = match File::open(dir.join("CMakeCache.txt")) {
             Ok(f) => f,
             Err(..) => return,
         };
-        f.read_to_string(&mut contents).unwrap();
+        let mut u8contents = Vec::new();
+        match f.read_to_end(&mut u8contents) {
+            Ok(f) => f,
+            Err(..) => return,
+        };
+        let contents = String::from_utf8_lossy(&u8contents);
         drop(f);
         for line in contents.lines() {
             if line.contains("CMAKE_HOME_DIRECTORY") && !line.contains(src) {
@@ -357,6 +413,13 @@ fn find_exe(path: &Path) -> PathBuf {
         .unwrap_or(path.to_owned())
 }
 
+fn getenv_unwrap(v: &str) -> String {
+    match env::var(v) {
+        Ok(s) => s,
+        Err(..) => fail(&format!("environment variable `{}` not defined", v)),
+    }
+}
+
 fn fail(s: &str) -> ! {
     panic!("\n{}\n\nbuild script failed, must exit now", s)
 }
