@@ -6,7 +6,7 @@ extern crate log;
 
 extern crate curl_sys as curl_ffi;
 
-#[cfg(unix)]
+#[cfg(all(unix, not(target_os = "macos")))]
 extern crate openssl_sys as openssl;
 
 pub use ffi::easy::ProgressCb;
