@@ -2,7 +2,7 @@ extern crate docopt;
 
 use docopt::Docopt;
 
-static USAGE: &'static str = "
+const USAGE: &'static str = "
 Naval Fate.
 
 Usage:
@@ -22,8 +22,9 @@ Options:
 ";
 
 fn main() {
+    let version = "1.2.3".to_owned();
     let args = Docopt::new(USAGE)
-                      .and_then(|dopt| dopt.parse())
+                      .and_then(|dopt| dopt.version(Some(version)).parse())
                       .unwrap_or_else(|e| e.exit());
     println!("{:?}", args);
 
@@ -34,4 +35,5 @@ fn main() {
     println!("  Speed: {}", args.get_str("--speed"));
     println!("  Drifting? {}", args.get_bool("--drifting"));
     println!("  Names: {:?}", args.get_vec("<name>"));
+    println!("  Command 'ship' invoked? {:?}", args.get_bool("ship"));
 }
