@@ -11,7 +11,7 @@ use docopt::Docopt;
 use rustc_serialize::{Decodable, Decoder};
 
 // Write the Docopt usage string.
-static USAGE: &'static str = "
+const USAGE: &'static str = "
 Rust's package manager
 
 Usage:
