@@ -28,7 +28,7 @@ use self::Error::{Usage, Argv, NoMatch, Decode, WithProgramUsage, Help, Version}
 /// ```no_run
 /// use docopt::Docopt;
 ///
-/// static USAGE: &'static str = "
+/// const USAGE: &'static str = "
 /// Usage: ...
 /// ";
 ///
@@ -241,9 +241,9 @@ impl Docopt {
     /// program. e.g., `["cp", "src", "dest"]` is right while `["src", "dest"]`
     /// is wrong.
     pub fn argv<I, S>(mut self, argv: I) -> Docopt
-               where I: Iterator<Item=S>, S: Into<String> {
+               where I: IntoIterator<Item=S>, S: AsRef<str> {
         self.argv = Some(
-            argv.skip(1).map(|s| s.into()).collect()
+            argv.into_iter().skip(1).map(|s| s.as_ref().to_owned()).collect()
         );
         self
     }
@@ -342,7 +342,7 @@ impl ArgvMap {
     /// # fn main() {
     /// use docopt::Docopt;
     ///
-    /// static USAGE: &'static str = "
+    /// const USAGE: &'static str = "
     /// Usage: cargo [options] (build | test)
     ///        cargo --help
     ///
@@ -652,8 +652,11 @@ impl Decoder {
     fn pop_key_val(&mut self) -> Result<(String, Value), Error> {
         let it = try!(self.pop());
         match it.val {
-            None => derr!("Could not find argument '{}' (from struct \
-                           field '{}').", it.key, it.struct_field),
+            None => derr!(
+                "Could not find argument '{}' (from struct field '{}').
+Note that each struct field must have the right key prefix, which must
+be one of `cmd_`, `flag_` or `arg_`.",
+                it.key, it.struct_field),
             Some(v) => Ok((it.key, v)),
         }
     }
@@ -668,10 +671,16 @@ impl Decoder {
         match v {
             Counted(n) => Ok(n),
             _ => {
-                match v.as_str().parse() {
-                    Err(_) => derr!("Could not decode '{}' to {} for '{}'.",
-                                    v.as_str(), expect, k),
-                    Ok(v) => Ok(v),
+                if v.as_str().trim().is_empty() {
+                    Ok(0)
+                } else {
+                    match v.as_str().parse() {
+                        Err(_) => {
+                            derr!("Could not decode '{}' to {} for '{}'.",
+                                  v.as_str(), expect, k)
+                        }
+                        Ok(v) => Ok(v),
+                    }
                 }
             }
         }
