@@ -114,6 +114,13 @@ impl Parser {
         }
     }
 
+    fn has_repeat(&self, atom: &Atom) -> bool {
+        match self.descs.find(atom) {
+            None => false,
+            Some(opts) => opts.repeats,
+        }
+    }
+
     fn parse(&mut self, doc: &str) -> Result<(), String> {
         let musage = regex!(r"(?s)(?i:usage):\s*(?P<prog>\S+)(?P<pats>.*?)(?:$|\n\s*\n)");
         let caps = match musage.captures(doc) {
@@ -148,12 +155,12 @@ impl Parser {
         }
 
         let mprog = format!(
-            "^(?:{})?\\s*(.*?)\\s*$", regex::quote(caps.name("prog").unwrap_or("")));
+            "^(?:{})?\\s*(.*?)\\s*$",
+            regex::quote(caps.name("prog").unwrap_or("")));
         let pats = Regex::new(&*mprog).unwrap();
 
         if caps.name("pats").unwrap_or("") == "" {
-            let pattern = try!(
-                PatParser::new(self, "").parse());
+            let pattern = try!(PatParser::new(self, "").parse());
             self.usages.push(pattern);
         } else {
             for line in caps.name("pats").unwrap_or("").lines() {
@@ -180,16 +187,25 @@ impl Parser {
         // after the flag or argument.
         let desc = regex!("  .*$").replace(desc, "");
         // Normalize `-x, --xyz` to `-x --xyz`.
-        let desc = regex!(r"([^-\s]), -").replace(&*desc, "$1 -");
+        let desc = regex!(r"([^-\s]), -").replace(&desc, "$1 -");
         let desc = desc.trim();
 
-        let rflags = regex!("(?:(?P<long>--[^ \t=]+)|(?P<short>-[^ \t=]+))\
-                             (?:(?: |=)(?P<arg>[^-]\\S*))?");
+        let rflags = regex!(r"(?x)
+            (?:(?P<long>--[^\x20\t=]+)|(?P<short>-[^\x20\t=]+))
+            (?:(?:\x20|=)(?P<arg>[^.-]\S*))?
+            (?P<repeated>\x20\.\.\.)?
+        ");
         let (mut short, mut long) = ("".to_string(), "".to_string());
         let mut has_arg = false;
         let mut last_end = 0;
+        let mut repeated = false;
         for flags in rflags.captures_iter(desc) {
             last_end = flags.pos(0).unwrap().1;
+            if !flags.name("repeated").unwrap_or("").is_empty() {
+                // If the "repeated" subcapture is not empty, then we have
+                // a valid repeated option.
+                repeated = true;
+            }
             let (s, l) = (
                 flags.name("short").unwrap_or(""),
                 flags.name("long").unwrap_or(""),
@@ -223,7 +239,7 @@ impl Parser {
             err!("Extraneous text '{}' in option description '{}'.",
                  &desc[last_end..], desc)
         }
-        try!(self.add_desc(&*short, &*long, has_arg));
+        try!(self.add_desc(&short, &long, has_arg, repeated));
         // Looking for default in this line must come after adding the
         // description, otherwise `parse_default` won't know which option
         // to assign it to.
@@ -262,17 +278,21 @@ impl Parser {
         Ok(())
     }
 
-    fn add_desc(&mut self, short: &str, long: &str, has_arg: bool)
-               -> Result<(), String> {
+    fn add_desc(
+        &mut self,
+        short: &str,
+        long: &str,
+        has_arg: bool,
+        repeated: bool,
+    ) -> Result<(), String> {
         assert!(!short.is_empty() || !long.is_empty());
         if !short.is_empty() && short.chars().count() != 2 {
             // It looks like the reference implementation just ignores
             // these lines.
             return Ok(());
-            // err!("Short flag '{}' is not of the form '-x'.", short);
         }
-        let mut opts = Options::new(false,
-                                    if has_arg { One(None) } else { Zero });
+        let mut opts = Options::new(
+            repeated, if has_arg { One(None) } else { Zero });
         opts.is_desc = true;
 
         if !short.is_empty() && !long.is_empty() {
@@ -447,7 +467,11 @@ impl<'a> PatParser<'a> {
         let stacked: String = self.cur()[1..].to_string();
         for (i, c) in stacked.chars().enumerate() {
             let atom = self.dopt.descs.resolve(&Short(c));
-            seq.push(PatAtom(atom.clone()));
+            let mut pat = PatAtom(atom.clone());
+            if self.dopt.has_repeat(&atom) {
+                pat = Pattern::repeat(pat);
+            }
+            seq.push(pat);
 
             // The only way for a short option to have an argument is if
             // it's specified in an option description.
@@ -477,7 +501,7 @@ impl<'a> PatParser<'a> {
         // here to group a short stack.
         if self.atis(0, "...") {
             self.next();
-            seq = seq.into_iter().map(|p| Repeat(Box::new(p))).collect();
+            seq = seq.into_iter().map(|p| Pattern::repeat(p)).collect();
         }
         Ok(seq)
     }
@@ -505,11 +529,16 @@ impl<'a> PatParser<'a> {
         }
         self.add_atom_ifnotexists(arg, &atom);
         self.next();
-        Ok(self.maybe_repeat(PatAtom(atom)))
+        let pat = if self.dopt.has_repeat(&atom) {
+            Pattern::repeat(PatAtom(atom))
+        } else {
+            PatAtom(atom)
+        };
+        Ok(self.maybe_repeat(pat))
     }
 
     fn next_flag_arg(&mut self, atom: &Atom) -> Result<(), String> {
-        try!(self.next_noeof(&*format!("argument for flag '{:?}'", atom)));
+        try!(self.next_noeof(&*format!("argument for flag '{}'", atom)));
         self.errif_invalid_flag_arg(atom, self.cur())
     }
 
@@ -553,7 +582,7 @@ impl<'a> PatParser<'a> {
     fn maybe_repeat(&mut self, pat: Pattern) -> Pattern {
         if self.atis(0, "...") {
             self.next();
-            Repeat(Box::new(pat))
+            Pattern::repeat(pat)
         } else {
             pat
         }
@@ -640,7 +669,11 @@ impl Pattern {
                     } else {
                         for atom in par.options_atoms().into_iter() {
                             if !all_atoms.contains(&atom) {
-                                ps.push(PatAtom(atom));
+                                if par.has_repeat(&atom) {
+                                    ps.push(Pattern::repeat(PatAtom(atom)));
+                                } else {
+                                    ps.push(PatAtom(atom));
+                                }
                             }
                         }
                     }
@@ -708,6 +741,13 @@ impl Pattern {
         let mut seen = HashSet::new();
         dotag(self, false, map, &mut seen);
     }
+
+    fn repeat(p: Pattern) -> Pattern {
+        match p {
+            p @ Repeat(_) => p,
+            p => Repeat(Box::new(p)),
+        }
+    }
 }
 
 impl Atom {
@@ -882,13 +922,13 @@ impl<'a> Argv<'a> {
                     err!("Flag '{}' cannot have an argument, but found '{:?}'.",
                          &atom, &arg)
                 } else if arg.is_none() && self.dopt.has_arg(&atom) {
-                    try!(self.next_noeof(&*format!("argument for flag '{:?}'",
+                    try!(self.next_noeof(&*format!("argument for flag '{}'",
                                                    &atom)));
                     arg = Some(self.cur().to_string());
                 }
                 self.flags.push(ArgvToken { atom: atom, arg: arg });
             } else {
-                if self.cur() == "--" {
+                if !seen_double_dash && self.cur() == "--" {
                     seen_double_dash = true;
                 } else {
                     // Yup, we *always* insert a positional argument, which
@@ -953,7 +993,7 @@ impl<'a> Argv<'a> {
         }
     }
     fn next_arg<'b>(&'b mut self, atom: &Atom) -> Result<&'b str, String> {
-        let expected = format!("argument for flag '{:?}'", atom);
+        let expected = format!("argument for flag '{}'", atom);
         try!(self.next_noeof(&*expected));
         Ok(self.cur())
     }
