@@ -13,7 +13,8 @@ use util::{self, Binding};
 /// The diff object that contains all individual file deltas.
 ///
 /// This is an opaque structure which will be allocated by one of the diff
-/// generator functions below (such as `Diff::tree_to_tree`).
+/// generator functions on the `Repository` structure (e.g. `diff_tree_to_tree`
+/// or other `diff_*` functions).
 pub struct Diff {
     raw: *mut raw::git_diff,
 }
@@ -76,132 +77,49 @@ pub struct DiffStats {
 type PrintCb<'a> = FnMut(DiffDelta, Option<DiffHunk>, DiffLine) -> bool + 'a;
 
 impl Diff {
-    /// Create a diff with the difference between two tree objects.
-    ///
-    /// This is equivalent to `git diff <old-tree> <new-tree>`
-    ///
-    /// The first tree will be used for the "old_file" side of the delta and the
-    /// second tree will be used for the "new_file" side of the delta.  You can
-    /// pass `None` to indicate an empty tree, although it is an error to pass
-    /// `None` for both the `old_tree` and `new_tree`.
+    /// Deprecated, use repo.diff_tree_to_tree(..) instead
+    #[doc(hidden)]
     pub fn tree_to_tree(repo: &Repository,
                         old_tree: Option<&Tree>,
                         new_tree: Option<&Tree>,
                         opts: Option<&mut DiffOptions>) -> Result<Diff, Error> {
-        let mut ret = 0 as *mut raw::git_diff;
-        unsafe {
-            try_call!(raw::git_diff_tree_to_tree(&mut ret,
-                                                 repo.raw(),
-                                                 old_tree.map(|s| s.raw()),
-                                                 new_tree.map(|s| s.raw()),
-                                                 opts.map(|s| s.raw())));
-            Ok(Binding::from_raw(ret))
-        }
+        repo.diff_tree_to_tree(old_tree, new_tree, opts)
     }
 
-    /// Create a diff between a tree and repository index.
-    ///
-    /// This is equivalent to `git diff --cached <treeish>` or if you pass
-    /// the HEAD tree, then like `git diff --cached`.
-    ///
-    /// The tree you pass will be used for the "old_file" side of the delta, and
-    /// the index will be used for the "new_file" side of the delta.
-    ///
-    /// If you pass `None` for the index, then the existing index of the `repo`
-    /// will be used.  In this case, the index will be refreshed from disk
-    /// (if it has changed) before the diff is generated.
-    ///
-    /// If the tree is `None`, then it is considered an empty tree.
+    /// Deprecated, use repo.diff_tree_to_index(..) instead
+    #[doc(hidden)]
     pub fn tree_to_index(repo: &Repository,
                          old_tree: Option<&Tree>,
                          index: Option<&Index>,
                          opts: Option<&mut DiffOptions>) -> Result<Diff, Error> {
-        let mut ret = 0 as *mut raw::git_diff;
-        unsafe {
-            try_call!(raw::git_diff_tree_to_index(&mut ret,
-                                                  repo.raw(),
-                                                  old_tree.map(|s| s.raw()),
-                                                  index.map(|s| s.raw()),
-                                                  opts.map(|s| s.raw())));
-            Ok(Binding::from_raw(ret))
-        }
+        repo.diff_tree_to_index(old_tree, index, opts)
     }
 
-    /// Create a diff between the repository index and the workdir directory.
-    ///
-    /// This matches the `git diff` command.  See the note below on
-    /// `tree_to_workdir` for a discussion of the difference between
-    /// `git diff` and `git diff HEAD` and how to emulate a `git diff <treeish>`
-    /// using libgit2.
-    ///
-    /// The index will be used for the "old_file" side of the delta, and the
-    /// working directory will be used for the "new_file" side of the delta.
-    ///
-    /// If you pass `None` for the index, then the existing index of the `repo`
-    /// will be used.  In this case, the index will be refreshed from disk
-    /// (if it has changed) before the diff is generated.
+    /// Deprecated, use repo.diff_index_to_workdir(..) instead
+    #[doc(hidden)]
     pub fn index_to_workdir(repo: &Repository,
                             index: Option<&Index>,
                             opts: Option<&mut DiffOptions>)
                             -> Result<Diff, Error> {
-        let mut ret = 0 as *mut raw::git_diff;
-        unsafe {
-            try_call!(raw::git_diff_index_to_workdir(&mut ret,
-                                                     repo.raw(),
-                                                     index.map(|s| s.raw()),
-                                                     opts.map(|s| s.raw())));
-            Ok(Binding::from_raw(ret))
-        }
+        repo.diff_index_to_workdir(index, opts)
     }
 
-    /// Create a diff between a tree and the working directory.
-    ///
-    /// The tree you provide will be used for the "old_file" side of the delta,
-    /// and the working directory will be used for the "new_file" side.
-    ///
-    /// This is not the same as `git diff <treeish>` or `git diff-index
-    /// <treeish>`.  Those commands use information from the index, whereas this
-    /// function strictly returns the differences between the tree and the files
-    /// in the working directory, regardless of the state of the index.  Use
-    /// `tree_to_workdir_with_index` to emulate those commands.
-    ///
-    /// To see difference between this and `tree_to_workdir_with_index`,
-    /// consider the example of a staged file deletion where the file has then
-    /// been put back into the working dir and further modified.  The
-    /// tree-to-workdir diff for that file is 'modified', but `git diff` would
-    /// show status 'deleted' since there is a staged delete.
-    ///
-    /// If `None` is passed for `tree`, then an empty tree is used.
+    /// Deprecated, use repo.diff_tree_to_tree(..) instead
+    #[doc(hidden)]
     pub fn tree_to_workdir(repo: &Repository,
                            old_tree: Option<&Tree>,
                            opts: Option<&mut DiffOptions>)
                            -> Result<Diff, Error> {
-        let mut ret = 0 as *mut raw::git_diff;
-        unsafe {
-            try_call!(raw::git_diff_tree_to_workdir(&mut ret,
-                                                    repo.raw(),
-                                                    old_tree.map(|s| s.raw()),
-                                                    opts.map(|s| s.raw())));
-            Ok(Binding::from_raw(ret))
-        }
+        repo.diff_tree_to_workdir(old_tree, opts)
     }
 
-    /// Create a diff between a tree and the working directory using index data
-    /// to account for staged deletes, tracked files, etc.
-    ///
-    /// This emulates `git diff <tree>` by diffing the tree to the index and
-    /// the index to the working directory and blending the results into a
-    /// single diff that includes staged deleted, etc.
+    /// Deprecated, use repo.diff_tree_to_workdir_with_index(..) instead
+    #[doc(hidden)]
     pub fn tree_to_workdir_with_index(repo: &Repository,
                                       old_tree: Option<&Tree>,
                                       opts: Option<&mut DiffOptions>)
                                       -> Result<Diff, Error> {
-        let mut ret = 0 as *mut raw::git_diff;
-        unsafe {
-            try_call!(raw::git_diff_tree_to_workdir_with_index(&mut ret,
-                    repo.raw(), old_tree.map(|s| s.raw()), opts.map(|s| s.raw())));
-            Ok(Binding::from_raw(ret))
-        }
+        repo.diff_tree_to_workdir_with_index(old_tree, opts)
     }
 
     /// Merge one diff into another.
@@ -334,6 +252,8 @@ impl<'a> DiffDelta<'a> {
             raw::GIT_DELTA_UNTRACKED => Delta::Untracked,
             raw::GIT_DELTA_TYPECHANGE => Delta::Typechange,
             raw::GIT_DELTA_UNREADABLE => Delta::Unreadable,
+            raw::GIT_DELTA_CONFLICTED => Delta::Conflicted,
+            n => panic!("unknown diff status: {}", n),
         }
     }
 
@@ -716,7 +636,7 @@ impl<'a> DiffLine<'a> {
     /// Content of this line as bytes.
     pub fn content(&self) -> &[u8] {
         unsafe {
-            slice::from_raw_parts((*self.raw).content,
+            slice::from_raw_parts((*self.raw).content as *const u8,
                                   (*self.raw).content_len as usize)
         }
     }
@@ -742,7 +662,7 @@ impl<'a> DiffLine<'a> {
             raw::GIT_DIFF_LINE_DEL_EOFNL => '<',
             raw::GIT_DIFF_LINE_FILE_HDR => 'F',
             raw::GIT_DIFF_LINE_HUNK_HDR => 'H',
-            raw::GIT_DIFF_LINE_LINE_BINARY => 'B',
+            raw::GIT_DIFF_LINE_BINARY => 'B',
             _ => ' ',
         }
     }
@@ -782,7 +702,10 @@ impl<'a> DiffHunk<'a> {
 
     /// Header text
     pub fn header(&self) -> &[u8] {
-        unsafe { &(*self.raw).header[..(*self.raw).header_len as usize] }
+        unsafe {
+            slice::from_raw_parts((*self.raw).header.as_ptr() as *const u8,
+                                  (*self.raw).header_len as usize)
+        }
     }
 }
 
@@ -1005,12 +928,10 @@ impl DiffFindOptions {
 
 #[cfg(test)]
 mod tests {
-    use super::Diff;
-
     #[test]
     fn smoke() {
         let (_td, repo) = ::test::repo_init();
-        let diff = Diff::tree_to_workdir(&repo, None, None).unwrap();
+        let diff = repo.diff_tree_to_workdir(None, None).unwrap();
         assert_eq!(diff.deltas().len(), 0);
         let stats = diff.stats().unwrap();
         assert_eq!(stats.insertions(), 0);
