@@ -67,7 +67,6 @@
 #![allow(trivial_numeric_casts, trivial_casts)]
 #![deny(missing_docs)]
 #![cfg_attr(test, deny(warnings))]
-#![cfg_attr(all(feature = "unstable", test, unix), feature(fs, fs_ext))]
 #![cfg_attr(feature = "unstable", feature(catch_panic))]
 
 extern crate libc;
@@ -88,8 +87,10 @@ pub use buf::Buf;
 pub use commit::{Commit, Parents};
 pub use config::{Config, ConfigEntry, ConfigEntries};
 pub use cred::{Cred, CredentialHelper};
+pub use describe::{Describe, DescribeFormatOptions, DescribeOptions};
 pub use diff::{Diff, DiffDelta, DiffFile, DiffOptions, Deltas};
 pub use diff::{DiffLine, DiffHunk, DiffStats, DiffFindOptions};
+pub use merge::{AnnotatedCommit, MergeOptions};
 pub use error::Error;
 pub use index::{Index, IndexEntry, IndexEntries, IndexMatchedPath};
 pub use note::{Note, Notes};
@@ -97,11 +98,10 @@ pub use object::Object;
 pub use oid::Oid;
 pub use pathspec::{Pathspec, PathspecMatchList, PathspecFailedEntries};
 pub use pathspec::{PathspecDiffEntries, PathspecEntries};
-pub use push::{Push, PushStatus};
 pub use reference::{Reference, References, ReferenceNames};
 pub use reflog::{Reflog, ReflogEntry, ReflogIter};
 pub use refspec::Refspec;
-pub use remote::{Remote, Refspecs, RemoteHead};
+pub use remote::{Remote, Refspecs, RemoteHead, FetchOptions, PushOptions};
 pub use remote_callbacks::{RemoteCallbacks, Credentials, TransferProgress};
 pub use remote_callbacks::{TransportMessage, Progress, UpdateTips};
 pub use repo::{Repository, RepositoryInitOptions};
@@ -113,6 +113,7 @@ pub use submodule::Submodule;
 pub use tag::Tag;
 pub use time::{Time, IndexTime};
 pub use tree::{Tree, TreeEntry, TreeIter};
+pub use treebuilder::TreeBuilder;
 pub use util::IntoCString;
 
 /// An enumeration of possible errors that can happen when working with a git
@@ -129,7 +130,7 @@ pub enum ErrorCode {
     Ambiguous,
     /// Output buffer too short to hold data
     BufSize,
-    /// Operation not allowed on bare repository
+    /// User-generated error
     User,
     /// Operation not allowed on bare repository
     BareRepo,
@@ -141,12 +142,96 @@ pub enum ErrorCode {
     NotFastForward,
     /// Name/ref spec was not in a valid format
     InvalidSpec,
-    /// Merge conflicts prevented operation
-    MergeConflict,
+    /// Checkout conflicts prevented operation
+    Conflict,
     /// Lock file prevented operation
     Locked,
     /// Reference value does not match expected
     Modified,
+    /// Authentication error
+    Auth,
+    /// Server certificate is invalid
+    Certificate,
+    /// Patch/merge has already been applied
+    Applied,
+    /// The requested peel operation is not possible
+    Peel,
+    /// Unexpected EOF
+    Eof,
+    /// Invalid operation or input
+    Invalid,
+    /// Uncommitted changes in index prevented operation
+    Uncommitted,
+    /// Operation was not valid for a directory,
+    Directory,
+}
+
+/// An enumeration of possible categories of things that can have
+/// errors when working with a git repository.
+#[derive(PartialEq, Eq, Clone, Debug, Copy)]
+pub enum ErrorClass {
+    /// Uncategorized
+    None,
+    /// Out of memory or insufficient allocated space
+    NoMemory,
+    /// Syscall or standard system library error
+    Os,
+    /// Invalid input
+    Invalid,
+    /// Error resolving or manipulating a reference
+    Reference,
+    /// ZLib failure
+    Zlib,
+    /// Bad repository state
+    Repository,
+    /// Bad configuration
+    Config,
+    /// Regex failure
+    Regex,
+    /// Bad object
+    Odb,
+    /// Invalid index data
+    Index,
+    /// Error creating or obtaining an object
+    Object,
+    /// Network error
+    Net,
+    /// Error manpulating a tag
+    Tag,
+    /// Invalid value in tree
+    Tree,
+    /// Hashing or packing error
+    Indexer,
+    /// Error from SSL
+    Ssl,
+    /// Error involing submodules
+    Submodule,
+    /// Threading error
+    Thread,
+    /// Error manipulating a stash
+    Stash,
+    /// Checkout failure
+    Checkout,
+    /// Invalid FETCH_HEAD
+    FetchHead,
+    /// Merge failure
+    Merge,
+    /// SSH failure
+    Ssh,
+    /// Error manipulating filters
+    Filter,
+    /// Error reverting commit
+    Revert,
+    /// Error from a user callback
+    Callback,
+    /// Error cherry-picking commit
+    CherryPick,
+    /// Can't describe object
+    Describe,
+    /// Error during rebase
+    Rebase,
+    /// Filesystem-related error
+    Filesystem,
 }
 
 /// A listing of the possible states that a repository can be in.
@@ -230,14 +315,53 @@ pub enum ConfigLevel {
     Highest,
 }
 
+/// Merge file favor options for `MergeOptions` instruct the file-level
+/// merging functionality how to deal with conflicting regions of the files.
+#[derive(PartialEq, Eq, Debug, Copy, Clone)]
+pub enum FileFavor {
+    /// When a region of a file is changed in both branches, a conflict will be
+    /// recorded in the index so that git_checkout can produce a merge file with
+    /// conflict markers in the working directory. This is the default.
+    Normal,
+    /// When a region of a file is changed in both branches, the file created
+    /// in the index will contain the "ours" side of any conflicting region.
+    /// The index will not record a conflict.
+    Ours,
+    /// When a region of a file is changed in both branches, the file created
+    /// in the index will contain the "theirs" side of any conflicting region.
+    /// The index will not record a conflict.
+    Theirs,
+    /// When a region of a file is changed in both branches, the file created
+    /// in the index will contain each unique line from each side, which has
+    /// the result of combining both files. The index will not record a conflict.
+    Union,
+}
+
 bitflags! {
     #[doc = "
 Orderings that may be specified for Revwalk iteration.
 "]
     flags Sort: u32 {
+        /// Sort the repository contents in no particular ordering.
+        ///
+        /// This sorting is arbitrary, implementation-specific, and subject to
+        /// change at any time. This is the default sorting for new walkers.
         const SORT_NONE = raw::GIT_SORT_NONE as u32,
+
+        /// Sort the repository contents in topological order (parents before
+        /// children).
+        ///
+        /// This sorting mode can be combined with time sorting.
         const SORT_TOPOLOGICAL = raw::GIT_SORT_TOPOLOGICAL as u32,
+
+        /// Sort the repository contents by commit time.
+        ///
+        /// This sorting mode can be combined with topological sorting.
         const SORT_TIME = raw::GIT_SORT_TIME as u32,
+
+        /// Iterate through the repository contents in reverse order.
+        ///
+        /// This sorting mode can be combined with any others.
         const SORT_REVERSE = raw::GIT_SORT_REVERSE as u32,
     }
 }
@@ -247,11 +371,19 @@ bitflags! {
 Types of credentials that can be requested by a credential callback.
 "]
     flags CredentialType: u32 {
+        #[allow(missing_docs)]
         const USER_PASS_PLAINTEXT = raw::GIT_CREDTYPE_USERPASS_PLAINTEXT as u32,
+        #[allow(missing_docs)]
         const SSH_KEY = raw::GIT_CREDTYPE_SSH_KEY as u32,
+        #[allow(missing_docs)]
+        const SSH_MEMORY = raw::GIT_CREDTYPE_SSH_MEMORY as u32,
+        #[allow(missing_docs)]
         const SSH_CUSTOM = raw::GIT_CREDTYPE_SSH_CUSTOM as u32,
+        #[allow(missing_docs)]
         const DEFAULT = raw::GIT_CREDTYPE_DEFAULT as u32,
+        #[allow(missing_docs)]
         const SSH_INTERACTIVE = raw::GIT_CREDTYPE_SSH_INTERACTIVE as u32,
+        #[allow(missing_docs)]
         const USERNAME = raw::GIT_CREDTYPE_USERNAME as u32,
     }
 }
@@ -261,10 +393,14 @@ bitflags! {
 Flags for APIs that add files matching pathspec
 "]
     flags IndexAddOption: u32 {
+        #[allow(missing_docs)]
         const ADD_DEFAULT = raw::GIT_INDEX_ADD_DEFAULT as u32,
+        #[allow(missing_docs)]
         const ADD_FORCE = raw::GIT_INDEX_ADD_FORCE as u32,
+        #[allow(missing_docs)]
         const ADD_DISABLE_PATHSPEC_MATCH =
                 raw::GIT_INDEX_ADD_DISABLE_PATHSPEC_MATCH as u32,
+        #[allow(missing_docs)]
         const ADD_CHECK_PATHSPEC = raw::GIT_INDEX_ADD_CHECK_PATHSPEC as u32,
     }
 }
@@ -274,8 +410,11 @@ bitflags! {
 Flags for the return value of `Repository::revparse`
 "]
     flags RevparseMode: u32 {
+        /// The spec targeted a single object
         const REVPARSE_SINGLE = raw::GIT_REVPARSE_SINGLE as u32,
+        /// The spec targeted a range of commits
         const REVPARSE_RANGE = raw::GIT_REVPARSE_RANGE as u32,
+        /// The spec used the `...` operator, which invokes special semantics.
         const REVPARSE_MERGE_BASE = raw::GIT_REVPARSE_MERGE_BASE as u32,
     }
 }
@@ -288,6 +427,7 @@ mod util;
 pub mod build;
 pub mod cert;
 pub mod string_array;
+pub mod oid_array;
 pub mod transport;
 
 mod blame;
@@ -297,14 +437,15 @@ mod buf;
 mod commit;
 mod config;
 mod cred;
+mod describe;
 mod diff;
+mod merge;
 mod error;
 mod index;
 mod note;
 mod object;
 mod oid;
 mod pathspec;
-mod push;
 mod reference;
 mod reflog;
 mod refspec;
@@ -319,6 +460,7 @@ mod submodule;
 mod tag;
 mod time;
 mod tree;
+mod treebuilder;
 
 fn init() {
     static INIT: Once = ONCE_INIT;
@@ -329,7 +471,9 @@ fn init() {
                 "couldn't initialize the libgit2 library: {}", r);
         assert_eq!(libc::atexit(shutdown), 0);
     });
-    extern fn shutdown() { unsafe { raw::git_libgit2_shutdown() } }
+    extern fn shutdown() {
+        unsafe { raw::git_libgit2_shutdown(); }
+    }
 }
 
 unsafe fn opt_bytes<'a, T>(_anchor: &'a T,
@@ -367,15 +511,11 @@ impl ObjectType {
     pub fn from_raw(raw: raw::git_otype) -> Option<ObjectType> {
         match raw {
             raw::GIT_OBJ_ANY => Some(ObjectType::Any),
-            raw::GIT_OBJ_BAD => None,
-            raw::GIT_OBJ__EXT1 => None,
             raw::GIT_OBJ_COMMIT => Some(ObjectType::Commit),
             raw::GIT_OBJ_TREE => Some(ObjectType::Tree),
             raw::GIT_OBJ_BLOB => Some(ObjectType::Blob),
             raw::GIT_OBJ_TAG => Some(ObjectType::Tag),
-            raw::GIT_OBJ__EXT2 => None,
-            raw::GIT_OBJ_OFS_DELTA => None,
-            raw::GIT_OBJ_REF_DELTA => None,
+            _ => None,
         }
     }
 
@@ -407,30 +547,50 @@ impl ConfigLevel {
             raw::GIT_CONFIG_LEVEL_LOCAL => ConfigLevel::Local,
             raw::GIT_CONFIG_LEVEL_APP => ConfigLevel::App,
             raw::GIT_CONFIG_HIGHEST_LEVEL => ConfigLevel::Highest,
+            n => panic!("unknown config level: {}", n),
         }
     }
 }
 
 bitflags! {
-    #[doc = "
-Flags for repository status
-"]
+    /// Status flags for a single file
+    ///
+    /// A combination of these values will be returned to indicate the status of
+    /// a file.  Status compares the working directory, the index, and the
+    /// current HEAD of the repository.  The `STATUS_INDEX_*` set of flags
+    /// represents the status of file in the index relative to the HEAD, and the
+    /// `STATUS_WT_*` set of flags represent the status of the file in the
+    /// working directory relative to the index.
     flags Status: u32 {
+        #[allow(missing_docs)]
         const STATUS_CURRENT = raw::GIT_STATUS_CURRENT as u32,
 
+        #[allow(missing_docs)]
         const STATUS_INDEX_NEW = raw::GIT_STATUS_INDEX_NEW as u32,
+        #[allow(missing_docs)]
         const STATUS_INDEX_MODIFIED = raw::GIT_STATUS_INDEX_MODIFIED as u32,
+        #[allow(missing_docs)]
         const STATUS_INDEX_DELETED = raw::GIT_STATUS_INDEX_DELETED as u32,
+        #[allow(missing_docs)]
         const STATUS_INDEX_RENAMED = raw::GIT_STATUS_INDEX_RENAMED as u32,
+        #[allow(missing_docs)]
         const STATUS_INDEX_TYPECHANGE = raw::GIT_STATUS_INDEX_TYPECHANGE as u32,
 
+        #[allow(missing_docs)]
         const STATUS_WT_NEW = raw::GIT_STATUS_WT_NEW as u32,
+        #[allow(missing_docs)]
         const STATUS_WT_MODIFIED = raw::GIT_STATUS_WT_MODIFIED as u32,
+        #[allow(missing_docs)]
         const STATUS_WT_DELETED = raw::GIT_STATUS_WT_DELETED as u32,
+        #[allow(missing_docs)]
         const STATUS_WT_TYPECHANGE = raw::GIT_STATUS_WT_TYPECHANGE as u32,
+        #[allow(missing_docs)]
         const STATUS_WT_RENAMED = raw::GIT_STATUS_WT_RENAMED as u32,
 
+        #[allow(missing_docs)]
         const STATUS_IGNORED = raw::GIT_STATUS_IGNORED as u32,
+        #[allow(missing_docs)]
+        const STATUS_CONFLICTED = raw::GIT_STATUS_CONFLICTED as u32,
     }
 }
 
@@ -439,14 +599,14 @@ bitflags! {
 Mode options for RepositoryInitOptions
 "]
     flags RepositoryInitMode: u32 {
-        #[doc = "Use permissions configured by umask - the default"]
+        /// Use permissions configured by umask - the default
         const REPOSITORY_INIT_SHARED_UMASK =
                     raw::GIT_REPOSITORY_INIT_SHARED_UMASK as u32,
-        #[doc = "Use `--shared=group` behavior, chmod'ing the new repo to be \
-                 group writable and \"g+sx\" for sticky group assignment"]
+        /// Use `--shared=group` behavior, chmod'ing the new repo to be
+        /// group writable and \"g+sx\" for sticky group assignment
         const REPOSITORY_INIT_SHARED_GROUP =
                     raw::GIT_REPOSITORY_INIT_SHARED_GROUP as u32,
-        #[doc = "Use `--shared=all` behavior, adding world readability."]
+        /// Use `--shared=all` behavior, adding world readability.
         const REPOSITORY_INIT_SHARED_ALL =
                     raw::GIT_REPOSITORY_INIT_SHARED_ALL as u32,
     }
@@ -475,6 +635,8 @@ pub enum Delta {
     Typechange,
     /// Entry is unreadable
     Unreadable,
+    /// Entry in the index is conflicted
+    Conflicted,
 }
 
 bitflags! {
@@ -521,48 +683,95 @@ Lastly, the following will only be returned for ignore "NONE".
 * WD_UNTRACKED      - wd contains untracked files
 "#]
     flags SubmoduleStatus: u32 {
+        #[allow(missing_docs)]
         const SUBMODULE_STATUS_IN_HEAD =
                 raw::GIT_SUBMODULE_STATUS_IN_HEAD as u32,
+        #[allow(missing_docs)]
         const SUBMODULE_STATUS_IN_INDEX =
                 raw::GIT_SUBMODULE_STATUS_IN_INDEX as u32,
+        #[allow(missing_docs)]
         const SUBMODULE_STATUS_IN_CONFIG =
                 raw::GIT_SUBMODULE_STATUS_IN_CONFIG as u32,
+        #[allow(missing_docs)]
         const SUBMODULE_STATUS_IN_WD =
                 raw::GIT_SUBMODULE_STATUS_IN_WD as u32,
+        #[allow(missing_docs)]
         const SUBMODULE_STATUS_INDEX_ADDED =
                 raw::GIT_SUBMODULE_STATUS_INDEX_ADDED as u32,
+        #[allow(missing_docs)]
         const SUBMODULE_STATUS_INDEX_DELETED =
                 raw::GIT_SUBMODULE_STATUS_INDEX_DELETED as u32,
+        #[allow(missing_docs)]
         const SUBMODULE_STATUS_INDEX_MODIFIED =
                 raw::GIT_SUBMODULE_STATUS_INDEX_MODIFIED as u32,
+        #[allow(missing_docs)]
         const SUBMODULE_STATUS_WD_UNINITIALIZED =
                 raw::GIT_SUBMODULE_STATUS_WD_UNINITIALIZED as u32,
+        #[allow(missing_docs)]
         const SUBMODULE_STATUS_WD_ADDED =
                 raw::GIT_SUBMODULE_STATUS_WD_ADDED as u32,
+        #[allow(missing_docs)]
         const SUBMODULE_STATUS_WD_DELETED =
                 raw::GIT_SUBMODULE_STATUS_WD_DELETED as u32,
+        #[allow(missing_docs)]
         const SUBMODULE_STATUS_WD_MODIFIED =
                 raw::GIT_SUBMODULE_STATUS_WD_MODIFIED as u32,
+        #[allow(missing_docs)]
         const SUBMODULE_STATUS_WD_INDEX_MODIFIED =
                 raw::GIT_SUBMODULE_STATUS_WD_INDEX_MODIFIED as u32,
+        #[allow(missing_docs)]
         const SUBMODULE_STATUS_WD_WD_MODIFIED =
                 raw::GIT_SUBMODULE_STATUS_WD_WD_MODIFIED as u32,
+        #[allow(missing_docs)]
         const SUBMODULE_STATUS_WD_UNTRACKED =
                 raw::GIT_SUBMODULE_STATUS_WD_UNTRACKED as u32,
     }
 
 }
 
+/// Submodule ignore values
+///
+/// These values represent settings for the `submodule.$name.ignore`
+/// configuration value which says how deeply to look at the working
+/// directory when getting the submodule status.
+pub enum SubmoduleIgnore {
+    /// Use the submodule's configuration
+    Unspecified,
+    /// Any change or untracked file is considered dirty
+    None,
+    /// Only dirty if tracked files have changed
+    Untracked,
+    /// Only dirty if HEAD has moved
+    Dirty,
+    /// Never dirty
+    All,
+}
+
 bitflags! {
-    #[doc = r#"
-"#]
+    /// ...
     flags PathspecFlags: u32 {
+        /// Use the default pathspec matching configuration.
         const PATHSPEC_DEFAULT = raw::GIT_PATHSPEC_DEFAULT as u32,
+        /// Force matching to ignore case, otherwise matching will use native
+        /// case sensitivity fo the platform filesystem.
         const PATHSPEC_IGNORE_CASE = raw::GIT_PATHSPEC_IGNORE_CASE as u32,
+        /// Force case sensitive matches, otherwise match will use the native
+        /// case sensitivity of the platform filesystem.
         const PATHSPEC_USE_CASE = raw::GIT_PATHSPEC_USE_CASE as u32,
+        /// Disable glob patterns and just use simple string comparison for
+        /// matching.
         const PATHSPEC_NO_GLOB = raw::GIT_PATHSPEC_NO_GLOB as u32,
+        /// Means that match functions return the error code `NotFound` if no
+        /// matches are found. By default no matches is a success.
         const PATHSPEC_NO_MATCH_ERROR = raw::GIT_PATHSPEC_NO_MATCH_ERROR as u32,
+        /// Means that the list returned should track which patterns matched
+        /// which files so that at the end of the match we can identify patterns
+        /// that did not match any files.
         const PATHSPEC_FIND_FAILURES = raw::GIT_PATHSPEC_FIND_FAILURES as u32,
+        /// Means that the list returned does not need to keep the actual
+        /// matching filenames. Use this to just test if there were any matches
+        /// at all or in combination with `PATHSPEC_FAILURES` to validate a
+        /// pathspec.
         const PATHSPEC_FAILURES_ONLY = raw::GIT_PATHSPEC_FAILURES_ONLY as u32,
     }
 }
@@ -584,16 +793,44 @@ pub enum DiffFormat {
 
 bitflags! {
     /// Formatting options for diff stats
-    flags DiffStatsFormat: u32 {
-        const DIFF_STATS_NONE = raw::GIT_DIFF_STATS_NONE as u32,
-        const DIFF_STATS_FULL = raw::GIT_DIFF_STATS_FULL as u32,
-        const DIFF_STATS_SHORT = raw::GIT_DIFF_STATS_SHORT as u32,
-        const DIFF_STATS_NUMBER = raw::GIT_DIFF_STATS_NUMBER as u32,
+    flags DiffStatsFormat: raw::git_diff_stats_format_t {
+        /// Don't generate any stats
+        const DIFF_STATS_NONE = raw::GIT_DIFF_STATS_NONE,
+        /// Equivalent of `--stat` in git
+        const DIFF_STATS_FULL = raw::GIT_DIFF_STATS_FULL,
+        /// Equivalent of `--shortstat` in git
+        const DIFF_STATS_SHORT = raw::GIT_DIFF_STATS_SHORT,
+        /// Equivalent of `--numstat` in git
+        const DIFF_STATS_NUMBER = raw::GIT_DIFF_STATS_NUMBER,
+        /// Extended header information such as creations, renames and mode
+        /// changes, equivalent of `--summary` in git
         const DIFF_STATS_INCLUDE_SUMMARY =
-            raw::GIT_DIFF_STATS_INCLUDE_SUMMARY as u32,
+            raw::GIT_DIFF_STATS_INCLUDE_SUMMARY,
     }
 }
 
+/// Automatic tag following options.
+pub enum AutotagOption {
+    /// Use the setting from the remote's configuration
+    Unspecified,
+    /// Ask the server for tags pointing to objects we're already downloading
+    Auto,
+    /// Don't ask for any tags beyond the refspecs
+    None,
+    /// Ask for all the tags
+    All,
+}
+
+/// Configuration for how pruning is done on a fetch
+pub enum FetchPrune {
+    /// Use the setting from the configuration
+    Unspecified,
+    /// Force pruning on
+    On,
+    /// Force pruning off
+    Off,
+}
+
 #[cfg(test)]
 mod tests {
     use super::ObjectType;
