@@ -8,10 +8,13 @@ use libc::{c_int, c_char, size_t, c_void, c_uint};
 use {raw, Revspec, Error, init, Object, RepositoryState, Remote, Buf};
 use {ResetType, Signature, Reference, References, Submodule, Blame, BlameOptions};
 use {Branches, BranchType, Index, Config, Oid, Blob, Branch, Commit, Tree};
+use {AnnotatedCommit, MergeOptions, SubmoduleIgnore, SubmoduleStatus};
 use {ObjectType, Tag, Note, Notes, StatusOptions, Statuses, Status, Revwalk};
-use {RevparseMode, RepositoryInitMode, Reflog, IntoCString};
+use {RevparseMode, RepositoryInitMode, Reflog, IntoCString, Describe};
+use {DescribeOptions, TreeBuilder, Diff, DiffOptions};
 use build::{RepoBuilder, CheckoutBuilder};
 use string_array::StringArray;
+use oid_array::OidArray;
 use util::{self, Binding};
 
 /// An owned git repository, representing all state associated with the
@@ -287,18 +290,11 @@ impl Repository {
     /// Create a remote with the given url and refspec in memory. You can use
     /// this when you have a URL instead of a remote's name. Note that anonymous
     /// remotes cannot be converted to persisted remotes.
-    pub fn remote_anonymous(&self,
-                            url: &str,
-                            fetch: Option<&str>) -> Result<Remote, Error> {
+    pub fn remote_anonymous(&self, url: &str) -> Result<Remote, Error> {
         let mut ret = 0 as *mut raw::git_remote;
         let url = try!(CString::new(url));
-        let fetch = match fetch {
-            Some(t) => Some(try!(CString::new(t))),
-            None => None,
-        };
         unsafe {
-            try_call!(raw::git_remote_create_anonymous(&mut ret, self.raw, url,
-                                                       fetch));
+            try_call!(raw::git_remote_create_anonymous(&mut ret, self.raw, url));
             Ok(Binding::from_raw(ret))
         }
     }
@@ -341,6 +337,63 @@ impl Repository {
         Ok(())
     }
 
+    /// Add a fetch refspec to the remote's configuration
+    ///
+    /// Add the given refspec to the fetch list in the configuration. No loaded
+    /// remote instances will be affected.
+    pub fn remote_add_fetch(&self, name: &str, spec: &str)
+                            -> Result<(), Error> {
+        let name = try!(CString::new(name));
+        let spec = try!(CString::new(spec));
+        unsafe {
+            try_call!(raw::git_remote_add_fetch(self.raw, name, spec));
+        }
+        Ok(())
+    }
+
+    /// Add a push refspec to the remote's configuration.
+    ///
+    /// Add the given refspec to the push list in the configuration. No
+    /// loaded remote instances will be affected.
+    pub fn remote_add_push(&self, name: &str, spec: &str)
+                           -> Result<(), Error> {
+        let name = try!(CString::new(name));
+        let spec = try!(CString::new(spec));
+        unsafe {
+            try_call!(raw::git_remote_add_push(self.raw, name, spec));
+        }
+        Ok(())
+    }
+
+    /// Set the remote's url in the configuration
+    ///
+    /// Remote objects already in memory will not be affected. This assumes
+    /// the common case of a single-url remote and will otherwise return an
+    /// error.
+    pub fn remote_set_url(&self, name: &str, url: &str) -> Result<(), Error> {
+        let name = try!(CString::new(name));
+        let url = try!(CString::new(url));
+        unsafe { try_call!(raw::git_remote_set_url(self.raw, name, url)); }
+        Ok(())
+    }
+
+    /// Set the remote's url for pushing in the configuration.
+    ///
+    /// Remote objects already in memory will not be affected. This assumes
+    /// the common case of a single-url remote and will otherwise return an
+    /// error.
+    ///
+    /// `None` indicates that it should be cleared.
+    pub fn remote_set_pushurl(&self, name: &str, pushurl: Option<&str>)
+                              -> Result<(), Error> {
+        let name = try!(CString::new(name));
+        let pushurl = try!(::opt_cstr(pushurl));
+        unsafe {
+            try_call!(raw::git_remote_set_pushurl(self.raw, name, pushurl));
+        }
+        Ok(())
+    }
+
     /// Sets the current head to the specified object and optionally resets
     /// the index and working tree to match.
     ///
@@ -681,9 +734,9 @@ impl Repository {
                   tree: &Tree,
                   parents: &[&Commit]) -> Result<Oid, Error> {
         let update_ref = try!(::opt_cstr(update_ref));
-        let parent_ptrs: Vec<*const raw::git_commit> =  parents.iter().map(|p| {
+        let mut parent_ptrs = parents.iter().map(|p| {
             p.raw() as *const raw::git_commit
-        }).collect();
+        }).collect::<Vec<_>>();
         let message = try!(CString::new(message));
         let mut raw = raw::git_oid { id: [0; raw::GIT_OID_RAWSZ] };
         unsafe {
@@ -696,7 +749,7 @@ impl Repository {
                                              message,
                                              tree.raw(),
                                              parents.len() as size_t,
-                                             parent_ptrs.as_ptr()));
+                                             parent_ptrs.as_mut_ptr()));
             Ok(Binding::from_raw(&raw as *const _))
         }
     }
@@ -785,6 +838,18 @@ impl Repository {
         }
     }
 
+    /// Creates a git_annotated_commit from the given reference.
+    pub fn reference_to_annotated_commit(&self, reference: &Reference)
+                                         -> Result<AnnotatedCommit, Error> {
+        let mut ret = 0 as *mut raw::git_annotated_commit;
+        unsafe {
+            try_call!(raw::git_annotated_commit_from_ref(&mut ret,
+                                                         self.raw(),
+                                                         reference.raw()));
+            Ok(AnnotatedCommit::from_raw(ret))
+        }
+    }
+
     /// Create a new action signature with default user and now timestamp.
     ///
     /// This looks up the user.name and user.email from the configuration and
@@ -836,6 +901,21 @@ impl Repository {
         }
     }
 
+    /// Get the status for a submodule.
+    ///
+    /// This looks at a submodule and tries to determine the status.  It
+    /// will return a combination of the `SubmoduleStatus` values.
+    pub fn submodule_status(&self, name: &str, ignore: SubmoduleIgnore)
+                            -> Result<SubmoduleStatus, Error> {
+        let mut ret = 0;
+        let name = try!(CString::new(name));
+        unsafe {
+            try_call!(raw::git_submodule_status(&mut ret, self.raw, name,
+                                                ignore));
+        }
+        Ok(SubmoduleStatus::from_bits_truncate(ret as u32))
+    }
+
     /// Lookup a reference to one of the objects in a repository.
     pub fn find_tree(&self, oid: Oid) -> Result<Tree, Error> {
         let mut raw = 0 as *mut raw::git_tree;
@@ -845,6 +925,24 @@ impl Repository {
         }
     }
 
+    /// Create a new TreeBuilder, optionally initialized with the
+    /// entries of the given Tree.
+    ///
+    /// The tree builder can be used to create or modify trees in memory and
+    /// write them as tree objects to the database.
+    pub fn treebuilder(&self, tree: Option<&Tree>) -> Result<TreeBuilder, Error> {
+        unsafe {
+            let mut ret = 0 as *mut raw::git_treebuilder;
+            let tree = match tree {
+                Some(tree) => tree.raw(),
+                None => 0 as *mut raw::git_tree,
+            };
+            try_call!(raw::git_treebuilder_new(&mut ret, self.raw, tree));
+            Ok(Binding::from_raw(ret))
+        }
+    }
+
+
     /// Create a new tag in the repository from an object
     ///
     /// A new reference will also be created pointing to this tag object. If
@@ -989,6 +1087,76 @@ impl Repository {
         Ok(())
     }
 
+    /// Merges the given commit(s) into HEAD, writing the results into the
+    /// working directory. Any changes are staged for commit and any conflicts
+    /// are written to the index. Callers should inspect the repository's index
+    /// after this completes, resolve any conflicts and prepare a commit.
+    ///
+    /// The merge performed uses the first common ancestor, unlike the
+    /// git-merge-recursive strategy, which may produce an artificial common
+    /// ancestor tree when there are multiple ancestors.
+    /// For compatibility with git, the repository is put into a merging state.
+    /// Once the commit is done (or if the uses wishes to abort), you should
+    /// clear this state by calling git_repository_state_cleanup().
+    pub fn merge(&self,
+                 annotated_commits: &[&AnnotatedCommit],
+                 merge_opts: Option<&mut MergeOptions>,
+                 checkout_opts: Option<&mut CheckoutBuilder>)
+                 -> Result<(), Error>
+    {
+        unsafe {
+            let mut raw_checkout_opts = mem::zeroed();
+            try_call!(raw::git_checkout_init_options(&mut raw_checkout_opts,
+                                raw::GIT_CHECKOUT_OPTIONS_VERSION));
+            if let Some(c) = checkout_opts {
+                c.configure(&mut raw_checkout_opts);
+            }
+
+            let mut commit_ptrs = annotated_commits.iter().map(|c| {
+                c.raw() as *const raw::git_annotated_commit
+            }).collect::<Vec<_>>();
+
+            try_call!(raw::git_merge(self.raw,
+                                     commit_ptrs.as_mut_ptr(),
+                                     annotated_commits.len() as size_t,
+                                     merge_opts.map(|o| o.raw())
+                                               .unwrap_or(0 as *const _),
+                                     &raw_checkout_opts));
+        }
+        Ok(())
+    }
+
+    /// Merge two commits, producing an index that reflects the result of
+    /// the merge. The index may be written as-is to the working directory or
+    /// checked out. If the index is to be converted to a tree, the caller
+    /// should resolve any conflicts that arose as part of the merge.
+    ///
+    /// The merge performed uses the first common ancestor, unlike the
+    /// git-merge-recursive strategy, which may produce an artificial common
+    /// ancestor tree when there are multiple ancestors.
+    ///
+    /// The returned index must be freed explicitly with git_index_free.
+    pub fn merge_commits(&self, our_commit: &Commit, their_commit: &Commit,
+                         opts: Option<&MergeOptions>) -> Result<Index, Error> {
+         let mut raw = 0 as *mut raw::git_index;
+        unsafe {
+            try_call!(raw::git_merge_commits(&mut raw, self.raw,
+                                             our_commit.raw(),
+                                             their_commit.raw(),
+                                             opts.map(|o| o.raw())));
+            Ok(Binding::from_raw(raw))
+        }
+    }
+
+    /// Remove all the metadata associated with an ongoing command like merge,
+    /// revert, cherry-pick, etc. For example: MERGE_HEAD, MERGE_MSG, etc.
+    pub fn cleanup_state(&self) -> Result<(), Error> {
+        unsafe {
+            try_call!(raw::git_repository_state_cleanup(self.raw));
+        }
+        Ok(())
+    }
+
     /// Add a note for an object
     ///
     /// The `notes_ref` argument is the canonical name of the reference to use,
@@ -1113,6 +1281,20 @@ impl Repository {
         }
     }
 
+    /// Find all merge bases between two commits
+    pub fn merge_bases(&self, one: Oid, two: Oid) -> Result<OidArray, Error> {
+        let mut arr = raw::git_oidarray {
+            ids: 0 as *mut raw::git_oid,
+            count: 0,
+        };
+        unsafe {
+            try_call!(raw::git_merge_bases(&mut arr, self.raw,
+                                          one.raw(), two.raw()));
+            Ok(Binding::from_raw(arr))
+        }
+    }
+
+
     /// Count the number of unique commits between two commit objects
     ///
     /// There is no need for branches containing the commits to have any
@@ -1174,6 +1356,149 @@ impl Repository {
         }
         Ok(())
     }
+
+    /// Describes a commit
+    ///
+    /// Performs a describe operation on the current commit and the worktree.
+    /// After performing a describe on HEAD, a status is run and description is
+    /// considered to be dirty if there are.
+    pub fn describe(&self, opts: &DescribeOptions) -> Result<Describe, Error> {
+        let mut ret = 0 as *mut _;
+        unsafe {
+            try_call!(raw::git_describe_workdir(&mut ret, self.raw, opts.raw()));
+            Ok(Binding::from_raw(ret))
+        }
+    }
+
+    /// Create a diff with the difference between two tree objects.
+    ///
+    /// This is equivalent to `git diff <old-tree> <new-tree>`
+    ///
+    /// The first tree will be used for the "old_file" side of the delta and the
+    /// second tree will be used for the "new_file" side of the delta.  You can
+    /// pass `None` to indicate an empty tree, although it is an error to pass
+    /// `None` for both the `old_tree` and `new_tree`.
+    pub fn diff_tree_to_tree(&self,
+                             old_tree: Option<&Tree>,
+                             new_tree: Option<&Tree>,
+                             opts: Option<&mut DiffOptions>)
+                             -> Result<Diff, Error> {
+        let mut ret = 0 as *mut raw::git_diff;
+        unsafe {
+            try_call!(raw::git_diff_tree_to_tree(&mut ret,
+                                                 self.raw(),
+                                                 old_tree.map(|s| s.raw()),
+                                                 new_tree.map(|s| s.raw()),
+                                                 opts.map(|s| s.raw())));
+            Ok(Binding::from_raw(ret))
+        }
+    }
+
+    /// Create a diff between a tree and repository index.
+    ///
+    /// This is equivalent to `git diff --cached <treeish>` or if you pass
+    /// the HEAD tree, then like `git diff --cached`.
+    ///
+    /// The tree you pass will be used for the "old_file" side of the delta, and
+    /// the index will be used for the "new_file" side of the delta.
+    ///
+    /// If you pass `None` for the index, then the existing index of the `repo`
+    /// will be used.  In this case, the index will be refreshed from disk
+    /// (if it has changed) before the diff is generated.
+    ///
+    /// If the tree is `None`, then it is considered an empty tree.
+    pub fn diff_tree_to_index(&self,
+                              old_tree: Option<&Tree>,
+                              index: Option<&Index>,
+                              opts: Option<&mut DiffOptions>)
+                              -> Result<Diff, Error> {
+        let mut ret = 0 as *mut raw::git_diff;
+        unsafe {
+            try_call!(raw::git_diff_tree_to_index(&mut ret,
+                                                  self.raw(),
+                                                  old_tree.map(|s| s.raw()),
+                                                  index.map(|s| s.raw()),
+                                                  opts.map(|s| s.raw())));
+            Ok(Binding::from_raw(ret))
+        }
+    }
+
+    /// Create a diff between the repository index and the workdir directory.
+    ///
+    /// This matches the `git diff` command.  See the note below on
+    /// `tree_to_workdir` for a discussion of the difference between
+    /// `git diff` and `git diff HEAD` and how to emulate a `git diff <treeish>`
+    /// using libgit2.
+    ///
+    /// The index will be used for the "old_file" side of the delta, and the
+    /// working directory will be used for the "new_file" side of the delta.
+    ///
+    /// If you pass `None` for the index, then the existing index of the `repo`
+    /// will be used.  In this case, the index will be refreshed from disk
+    /// (if it has changed) before the diff is generated.
+    pub fn diff_index_to_workdir(&self,
+                                 index: Option<&Index>,
+                                 opts: Option<&mut DiffOptions>)
+                                 -> Result<Diff, Error> {
+        let mut ret = 0 as *mut raw::git_diff;
+        unsafe {
+            try_call!(raw::git_diff_index_to_workdir(&mut ret,
+                                                     self.raw(),
+                                                     index.map(|s| s.raw()),
+                                                     opts.map(|s| s.raw())));
+            Ok(Binding::from_raw(ret))
+        }
+    }
+
+    /// Create a diff between a tree and the working directory.
+    ///
+    /// The tree you provide will be used for the "old_file" side of the delta,
+    /// and the working directory will be used for the "new_file" side.
+    ///
+    /// This is not the same as `git diff <treeish>` or `git diff-index
+    /// <treeish>`.  Those commands use information from the index, whereas this
+    /// function strictly returns the differences between the tree and the files
+    /// in the working directory, regardless of the state of the index.  Use
+    /// `tree_to_workdir_with_index` to emulate those commands.
+    ///
+    /// To see difference between this and `tree_to_workdir_with_index`,
+    /// consider the example of a staged file deletion where the file has then
+    /// been put back into the working dir and further modified.  The
+    /// tree-to-workdir diff for that file is 'modified', but `git diff` would
+    /// show status 'deleted' since there is a staged delete.
+    ///
+    /// If `None` is passed for `tree`, then an empty tree is used.
+    pub fn diff_tree_to_workdir(&self,
+                                old_tree: Option<&Tree>,
+                                opts: Option<&mut DiffOptions>)
+                                -> Result<Diff, Error> {
+        let mut ret = 0 as *mut raw::git_diff;
+        unsafe {
+            try_call!(raw::git_diff_tree_to_workdir(&mut ret,
+                                                    self.raw(),
+                                                    old_tree.map(|s| s.raw()),
+                                                    opts.map(|s| s.raw())));
+            Ok(Binding::from_raw(ret))
+        }
+    }
+
+    /// Create a diff between a tree and the working directory using index data
+    /// to account for staged deletes, tracked files, etc.
+    ///
+    /// This emulates `git diff <tree>` by diffing the tree to the index and
+    /// the index to the working directory and blending the results into a
+    /// single diff that includes staged deleted, etc.
+    pub fn diff_tree_to_workdir_with_index(&self,
+                                           old_tree: Option<&Tree>,
+                                           opts: Option<&mut DiffOptions>)
+                                           -> Result<Diff, Error> {
+        let mut ret = 0 as *mut raw::git_diff;
+        unsafe {
+            try_call!(raw::git_diff_tree_to_workdir_with_index(&mut ret,
+                    self.raw(), old_tree.map(|s| s.raw()), opts.map(|s| s.raw())));
+            Ok(Binding::from_raw(ret))
+        }
+    }
 }
 
 impl Binding for Repository {
@@ -1345,6 +1670,7 @@ impl RepositoryInitOptions {
 #[cfg(test)]
 mod tests {
     use std::fs;
+    use std::path::Path;
     use tempdir::TempDir;
     use {Repository, Oid, ObjectType, ResetType};
     use build::CheckoutBuilder;
@@ -1504,6 +1830,88 @@ mod tests {
         assert_eq!(repo.head().unwrap().target().unwrap(), master_oid);
     }
 
+    /// create an octopus:
+    ///   /---o2-o4
+    /// o1      X
+    ///   \---o3-o5
+    /// and checks that the merge bases of (o4,o5) are (o2,o3)
+    #[test]
+    fn smoke_merge_bases() {
+        let (_td, repo) = graph_repo_init();
+        let sig = repo.signature().unwrap();
+
+        // let oid1 = head
+        let oid1 = repo.head().unwrap().target().unwrap();
+        let commit1 = repo.find_commit(oid1).unwrap();
+        println!("created oid1 {:?}", oid1);
+
+        repo.branch("branch_a", &commit1, true).unwrap();
+        repo.branch("branch_b", &commit1, true).unwrap();
+
+        // create commit oid2 on branchA
+        let mut index = repo.index().unwrap();
+        let p = Path::new(repo.workdir().unwrap()).join("file_a");
+        println!("using path {:?}", p);
+        fs::File::create(&p).unwrap();
+        index.add_path(Path::new("file_a")).unwrap();
+        let id_a = index.write_tree().unwrap();
+        let tree_a = repo.find_tree(id_a).unwrap();
+        let oid2 = repo.commit(Some("refs/heads/branch_a"), &sig, &sig,
+                               "commit 2", &tree_a, &[&commit1]).unwrap();
+        let commit2 = repo.find_commit(oid2).unwrap();
+        println!("created oid2 {:?}", oid2);
+
+        // create commit oid3 on branchB
+        let mut index = repo.index().unwrap();
+        let p = Path::new(repo.workdir().unwrap()).join("file_b");
+        fs::File::create(&p).unwrap();
+        index.add_path(Path::new("file_b")).unwrap();
+        let id_b = index.write_tree().unwrap();
+        let tree_b = repo.find_tree(id_b).unwrap();
+        let oid3 = repo.commit(Some("refs/heads/branch_b"), &sig, &sig,
+                               "commit 3", &tree_b, &[&commit1]).unwrap();
+        let commit3 = repo.find_commit(oid3).unwrap();
+        println!("created oid3 {:?}", oid3);
+
+        // create merge commit oid4 on branchA with parents oid2 and oid3
+        //let mut index4 = repo.merge_commits(&commit2, &commit3, None).unwrap();
+        repo.set_head("refs/heads/branch_a").unwrap();
+        repo.checkout_head(None).unwrap();
+        let oid4 = repo.commit(Some("refs/heads/branch_a"), &sig, &sig,
+                               "commit 4", &tree_a,
+                               &[&commit2, &commit3]).unwrap();
+        //index4.write_tree_to(&repo).unwrap();
+        println!("created oid4 {:?}", oid4);
+
+        // create merge commit oid5 on branchB with parents oid2 and oid3
+        //let mut index5 = repo.merge_commits(&commit3, &commit2, None).unwrap();
+        repo.set_head("refs/heads/branch_b").unwrap();
+        repo.checkout_head(None).unwrap();
+        let oid5 = repo.commit(Some("refs/heads/branch_b"), &sig, &sig,
+                               "commit 5", &tree_a,
+                               &[&commit3, &commit2]).unwrap();
+        //index5.write_tree_to(&repo).unwrap();
+        println!("created oid5 {:?}", oid5);
+
+        // merge bases of (oid4,oid5) should be (oid2,oid3)
+        let merge_bases = repo.merge_bases(oid4, oid5).unwrap();
+        let mut found_oid2 = false;
+        let mut found_oid3 = false;
+        for mg in merge_bases.iter() {
+            println!("found merge base {:?}", mg);
+            if mg == &oid2 {
+                found_oid2 = true;
+            } else if mg == &oid3 {
+                found_oid3 = true;
+            } else {
+                assert!(false);
+            }
+        }
+        assert!(found_oid2);
+        assert!(found_oid3);
+	assert_eq!(merge_bases.len(), 2);
+    }
+
     #[test]
     fn smoke_revparse_ext() {
         let (_td, repo) = graph_repo_init();
