@@ -93,17 +93,16 @@ impl CharRanges {
 
     /// Tests whether the given input character matches this instruction.
     #[inline(always)] // About ~5-15% more throughput then `#[inline]`
-    pub fn matches(&self, c: Char) -> Option<usize> {
+    pub fn matches(&self, c: Char) -> bool {
         // This speeds up the `match_class_unicode` benchmark by checking
         // some common cases quickly without binary search. e.g., Matching
         // a Unicode class on predominantly ASCII text.
-        for i in 0..cmp::min(self.ranges.len(), 4) {
-            let r = self.ranges[i];
+        for r in self.ranges.iter().take(4) {
             if c < r.0 {
-                return None;
+                return false;
             }
             if c <= r.1 {
-                return Some(i);
+                return true;
             }
         }
         self.ranges.binary_search_by(|r| {
@@ -114,7 +113,7 @@ impl CharRanges {
             } else {
                 Ordering::Equal
             }
-        }).ok()
+        }).is_ok()
     }
 }
 
@@ -283,7 +282,7 @@ impl Program {
         // First, look for a standard literal prefix---this includes things
         // like `a+` and `[0-9]+`, but not `a|b`.
         let (ps, complete) = self.literals(self.skip(1));
-        if ps.len() > 0 {
+        if !ps.is_empty() {
             self.prefixes = Prefix::new(ps);
             self.prefixes_complete = complete;
             return;
@@ -301,8 +300,8 @@ impl Program {
         let mut stack = vec![self.skip(1)];
         while let Some(mut pc) = stack.pop() {
             pc = self.skip(pc);
-            match &self.insts[pc] {
-                &Inst::Split(x, y) => { stack.push(y); stack.push(x); }
+            match self.insts[pc] {
+                Inst::Split(x, y) => { stack.push(y); stack.push(x); }
                 _ => {
                     let (alt_prefixes, complete) = self.literals(pc);
                     if alt_prefixes.is_empty() {
@@ -393,7 +392,7 @@ impl Program {
                 _ => { complete = self.leads_to_match(pc); break }
             }
         }
-        if alts[0].len() == 0 {
+        if alts[0].is_empty() {
             (vec![], false)
         } else {
             (alts, complete)
@@ -444,9 +443,8 @@ impl Clone for Program {
 fn num_captures(insts: &[Inst]) -> usize {
     let mut n = 0;
     for inst in insts {
-        match *inst {
-            Inst::Save(c) => n = cmp::max(n, c+1),
-            _ => {}
+        if let Inst::Save(c) = *inst {
+            n = cmp::max(n, c+1)
         }
     }
     // There's exactly 2 Save slots for every capture.
