@@ -12,6 +12,7 @@ use std::borrow::Cow;
 use std::collections::HashMap;
 use std::collections::hash_map::Iter;
 use std::fmt;
+use std::ops::Index;
 #[cfg(feature = "pattern")]
 use std::str::pattern::{Pattern, Searcher, SearchStep};
 use std::str::FromStr;
@@ -377,6 +378,11 @@ impl Regex {
     /// assert_eq!(caps.at(1), Some("Citizen Kane"));
     /// assert_eq!(caps.at(2), Some("1941"));
     /// assert_eq!(caps.at(0), Some("'Citizen Kane' (1941)"));
+    /// // You can also access the groups by index using the Index notation.
+    /// // Note that this will panic on an invalid index.
+    /// assert_eq!(&caps[1], "Citizen Kane");
+    /// assert_eq!(&caps[2], "1941");
+    /// assert_eq!(&caps[0], "'Citizen Kane' (1941)");
     /// # }
     /// ```
     ///
@@ -395,15 +401,21 @@ impl Regex {
     /// assert_eq!(caps.name("title"), Some("Citizen Kane"));
     /// assert_eq!(caps.name("year"), Some("1941"));
     /// assert_eq!(caps.at(0), Some("'Citizen Kane' (1941)"));
+    /// // You can also access the groups by name using the Index notation.
+    /// // Note that this will panic on an invalid group name.
+    /// assert_eq!(&caps["title"], "Citizen Kane");
+    /// assert_eq!(&caps["year"], "1941");
+    /// assert_eq!(&caps[0], "'Citizen Kane' (1941)");
+    ///
     /// # }
     /// ```
     ///
     /// Here we name the capture groups, which we can access with the `name`
-    /// method. Note that the named capture groups are still accessible with
-    /// `at`.
+    /// method or the `Index` notation with a `&str`. Note that the named capture groups
+    /// are still accessible with `at` or the `Index` notation with a `usize`.
     ///
     /// The `0`th capture group is always unnamed, so it must always be
-    /// accessed with `at(0)`.
+    /// accessed with `at(0)` or `[0]`.
     pub fn captures<'t>(&self, text: &'t str) -> Option<Captures<'t>> {
         let mut caps = self.alloc_captures();
         if exec(self, &mut caps, text, 0) {
@@ -625,22 +637,23 @@ impl Regex {
     }
 
     /// Returns the original string of this regex.
-    pub fn as_str<'a>(&'a self) -> &'a str {
+    pub fn as_str(&self) -> &str {
         match *self {
             Regex::Dynamic(Program { ref original, .. }) => original,
             Regex::Native(ExNative { ref original, .. }) => original,
         }
     }
 
-    #[doc(hidden)]
-    pub fn names_iter<'a>(&'a self) -> NamesIter<'a> {
+    /// Returns an iterator over the capture names.
+    pub fn capture_names(&self) -> CaptureNames {
         match *self {
-            Regex::Native(ref n) => NamesIter::Native(n.names.iter()),
-            Regex::Dynamic(ref d) => NamesIter::Dynamic(d.cap_names.iter())
+            Regex::Native(ref n) => CaptureNames::Native(n.names.iter()),
+            Regex::Dynamic(ref d) => CaptureNames::Dynamic(d.cap_names.iter())
         }
     }
 
-    fn names_len(&self) -> usize {
+    /// Returns the number of captures.
+    pub fn captures_len(&self) -> usize {
         match *self {
             Regex::Native(ref n) => n.names.len(),
             Regex::Dynamic(ref d) => d.cap_names.len()
@@ -655,20 +668,34 @@ impl Regex {
     }
 }
 
-pub enum NamesIter<'a> {
-    Native(::std::slice::Iter<'a, Option<&'static str>>),
-    Dynamic(::std::slice::Iter<'a, Option<String>>)
+/// Yields the names of all possible captures.
+/// `None` indicates an unnamed capture; the first element
+/// (capture 0, the whole matched region) is always unnamed.
+///
+/// `'r` is the lifetime of the compiled expression.
+pub enum CaptureNames<'r> {
+    #[doc(hidden)]
+    Native(::std::slice::Iter<'r, Option<&'static str>>),
+    #[doc(hidden)]
+    Dynamic(::std::slice::Iter<'r, Option<String>>)
 }
 
-impl<'a> Iterator for NamesIter<'a> {
-    type Item=Option<String>;
+impl<'r> Iterator for CaptureNames<'r> {
+    type Item=Option<&'r str>;
 
-    fn next(&mut self) -> Option<Option<String>> {
+    fn next(&mut self) -> Option<Option<&'r str>> {
         match *self {
-            NamesIter::Native(ref mut i) =>
-                i.next().map(|x| x.map(|s| s.to_owned())),
-            NamesIter::Dynamic(ref mut i) =>
-                i.next().map(|x| x.as_ref().map(|s| s.to_owned())),
+            CaptureNames::Native(ref mut i) =>
+                i.next().cloned(),
+            CaptureNames::Dynamic(ref mut i) =>
+                i.next().as_ref().map(|o| o.as_ref().map(|s| s.as_ref())),
+        }
+    }
+
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        match *self {
+            CaptureNames::Native(ref i)  => i.size_hint(),
+            CaptureNames::Dynamic(ref i) => i.size_hint(),
         }
     }
 }
@@ -689,18 +716,18 @@ pub trait Replacer {
     ///
     /// The `'a` lifetime refers to the lifetime of a borrowed string when
     /// a new owned string isn't needed (e.g., for `NoExpand`).
-    fn reg_replace<'a>(&'a mut self, caps: &Captures) -> Cow<'a, str>;
+    fn reg_replace(&mut self, caps: &Captures) -> Cow<str>;
 
     /// Returns a possibly owned string that never needs expansion.
-    fn no_expand<'a>(&'a mut self) -> Option<Cow<'a, str>> { None }
+    fn no_expand(&mut self) -> Option<Cow<str>> { None }
 }
 
 impl<'t> Replacer for NoExpand<'t> {
-    fn reg_replace<'a>(&'a mut self, _: &Captures) -> Cow<'a, str> {
+    fn reg_replace(&mut self, _: &Captures) -> Cow<str> {
         self.0.into()
     }
 
-    fn no_expand<'a>(&'a mut self) -> Option<Cow<'a, str>> {
+    fn no_expand(&mut self) -> Option<Cow<str>> {
         Some(self.0.into())
     }
 }
@@ -710,12 +737,11 @@ impl<'t> Replacer for &'t str {
         caps.expand(*self).into()
     }
 
-    fn no_expand<'a>(&'a mut self) -> Option<Cow<'a, str>> {
-        let re = Regex::new(REPLACE_EXPAND).unwrap();
-        if !re.is_match(self) {
-            Some((*self).into())
-        } else {
-            None
+    fn no_expand(&mut self) -> Option<Cow<str>> {
+        // if there is a $ there may be an expansion
+        match self.find('$') {
+            Some(_) => None,
+            None => Some((*self).into()),
         }
     }
 }
@@ -813,13 +839,13 @@ impl<'t> Captures<'t> {
         locs: Vec<Option<usize>>,
     ) -> Captures<'t> {
         let named =
-            if re.names_len() == 0 {
+            if re.captures_len() == 0 {
                 None
             } else {
                 let mut named = HashMap::new();
-                for (i, name) in re.names_iter().enumerate() {
+                for (i, name) in re.capture_names().enumerate() {
                     if let Some(name) = name {
-                        named.insert(name, i);
+                        named.insert(name.to_owned(), i);
                     }
                 }
                 Some(named)
@@ -909,8 +935,8 @@ impl<'t> Captures<'t> {
             let before = refs.name("before").unwrap_or("");
             let name = refs.name("name").unwrap_or("");
             format!("{}{}", before, match name.parse::<usize>() {
-                Err(_) => self.name(name).unwrap_or("").to_string(),
-                Ok(i) => self.at(i).unwrap_or("").to_string(),
+                Err(_) => self.name(name).unwrap_or("").to_owned(),
+                Ok(i) => self.at(i).unwrap_or("").to_owned(),
             })
         });
         let re = Regex::new(r"\$\$").unwrap();
@@ -926,6 +952,37 @@ impl<'t> Captures<'t> {
     pub fn is_empty(&self) -> bool { self.len() == 0 }
 }
 
+/// Get a group by index.
+///
+/// # Panics
+/// If there is no group at the given index.
+impl<'t> Index<usize> for Captures<'t> {
+
+    type Output = str;
+
+    fn index(&self, i: usize) -> &str {
+        self.at(i).unwrap_or_else(|| panic!("no group at index '{}'", i))
+    }
+
+}
+
+/// Get a group by name.
+///
+/// # Panics
+/// If there is no group named by the given value.
+impl<'t> Index<&'t str> for Captures<'t> {
+
+    type Output = str;
+
+    fn index<'a>(&'a self, name: &str) -> &'a str {
+        match self.name(name) {
+            None => panic!("no group named '{}'", name),
+            Some(ref s) => s,
+        }
+    }
+
+}
+
 /// An iterator over capture groups for a particular match of a regular
 /// expression.
 ///
@@ -985,7 +1042,7 @@ impl<'t> Iterator for SubCapturesNamed<'t> {
     type Item = (&'t str, Option<&'t str>);
 
     fn next(&mut self) -> Option<(&'t str, Option<&'t str>)> {
-        match self.inner.as_mut().map(|it| it.next()).unwrap_or(None) {
+        match self.inner.as_mut().map_or(None, |it| it.next()) {
             Some((name, pos)) => Some((name, self.caps.at(*pos))),
             None => None
         }
@@ -1146,3 +1203,63 @@ fn exec(re: &Regex, caps: &mut CaptureIdxs, text: &str, start: usize) -> bool {
         Regex::Dynamic(ref prog) => prog.exec(caps, text, start),
     }
 }
+
+#[cfg(test)]
+mod test {
+    use super::{NoExpand, Regex};
+
+    #[test]
+    fn test_simple_expand() {
+        let re = Regex::new(r"(\w) (\w)").unwrap();
+        assert_eq!(re.replace_all("a b", "$2 $1"), "b a");
+    }
+
+    #[test]
+    fn test_literal_dollar() {
+        let re = Regex::new(r"(\w+) (\w+)").unwrap();
+        assert_eq!(re.replace_all("a b", "$1"), "a");
+        assert_eq!(re.replace_all("a b", "$$1"), "$1");  // $$ should become a $
+        assert_eq!(re.replace_all("a b", "$2 $$c $1"), "b $c a");
+    }
+
+    #[test]
+    fn test_no_expand() {
+        let re = Regex::new(r"(\w+)").unwrap();
+        assert_eq!(re.replace_all("a", NoExpand("$$1")), "$$1");
+        assert_eq!(re.replace_all("a", NoExpand("$1")), "$1");
+    }
+
+    #[test]
+    fn test_capture_names() {
+        let re = Regex::new(r"(.)(?P<a>.)").unwrap();
+        assert_eq!(re.capture_names().size_hint(), (3, Some(3)));
+        assert_eq!(re.capture_names().collect::<Vec<_>>(), [None, None, Some("a")]);
+    }
+
+    #[test]
+    fn test_cap_index() {
+        let re = Regex::new(r"^(?P<name>.+)$").unwrap();
+        let cap = re.captures("abc").unwrap();
+        assert_eq!(&cap[0], "abc");
+        assert_eq!(&cap[1], "abc");
+        assert_eq!(&cap["name"], "abc");
+    }
+
+    #[test]
+    #[should_panic]
+    #[cfg_attr(all(target_env = "msvc", target_pointer_width = "32"), ignore)]
+    fn test_cap_index_panic_usize() {
+        let re = Regex::new(r"^(?P<name>.+)$").unwrap();
+        let cap = re.captures("abc").unwrap();
+        let _ = cap[2];
+    }
+
+    #[test]
+    #[should_panic]
+    #[cfg_attr(all(target_env = "msvc", target_pointer_width = "32"), ignore)]
+    fn test_cap_index_panic_name() {
+        let re = Regex::new(r"^(?P<name>.+)$").unwrap();
+        let cap = re.captures("abc").unwrap();
+        let _ = cap["bad name"];
+    }
+}
