@@ -197,7 +197,7 @@ pub trait Encodable {
     fn encode<S: Encoder>(&self, s: &mut S) -> Result<(), S::Error>;
 }
 
-pub trait Decodable {
+pub trait Decodable: Sized {
     fn decode<D: Decoder>(d: &mut D) -> Result<Self, D::Error>;
 }
 
@@ -512,8 +512,8 @@ impl<T:Decodable> Decodable for Option<T> {
 }
 
 impl<T> Encodable for PhantomData<T> {
-    fn encode<S: Encoder>(&self, _s: &mut S) -> Result<(), S::Error> {
-        Ok(())
+    fn encode<S: Encoder>(&self, s: &mut S) -> Result<(), S::Error> {
+        s.emit_nil()
     }
 }
 
