@@ -8,10 +8,12 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-//! The `version` module gives you tools to create and compare SemVer-compliant versions.
+//! The `version` module gives you tools to create and compare SemVer-compliant
+//! versions.
 
 use std::ascii::AsciiExt;
 use std::cmp::{self, Ordering};
+use std::error::Error;
 use std::fmt;
 use std::hash;
 
@@ -20,7 +22,8 @@ use self::ParseError::{GenericFailure, IncorrectParse, NonAsciiIdentifier};
 
 /// An identifier in the pre-release or build metadata.
 ///
-/// See sections 9 and 10 of the spec for more about pre-release identifers and build metadata.
+/// See sections 9 and 10 of the spec for more about pre-release identifers and
+/// build metadata.
 #[derive(Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug)]
 pub enum Identifier {
     /// An identifier that's solely numbers.
@@ -57,8 +60,8 @@ pub struct Version {
     pub build: Vec<Identifier>,
 }
 
-/// A `ParseError` is returned as the `Err` side of a `Result` when a version is attempted
-/// to be parsed.
+/// A `ParseError` is returned as the `Err` side of a `Result` when a version is
+/// attempted to be parsed.
 #[derive(Clone,PartialEq,Debug,PartialOrd)]
 pub enum ParseError {
     /// All identifiers must be ASCII.
@@ -88,6 +91,43 @@ impl Version {
             None => Err(GenericFailure)
         }
     }
+
+    /// Clears the build metadata
+    fn clear_metadata(&mut self) {
+        self.build = Vec::new();
+        self.pre = Vec::new();
+    }
+
+    /// Increments the patch number for this Version (Must be mutable)
+    pub fn increment_patch(&mut self) {
+        self.patch += 1;
+        self.clear_metadata();
+    }
+
+    /// Increments the minor version number for this Version (Must be mutable)
+    ///
+    /// As instructed by section 7 of the spec, the patch number is reset to 0.
+    pub fn increment_minor(&mut self) {
+        self.minor += 1;
+        self.patch = 0;
+        self.clear_metadata();
+    }
+
+    /// Increments the major version number for this Version (Must be mutable)
+    ///
+    /// As instructed by section 8 of the spec, the minor and patch numbers are
+    /// reset to 0
+    pub fn increment_major(&mut self) {
+        self.major += 1;
+        self.minor = 0;
+        self.patch = 0;
+        self.clear_metadata();
+    }
+
+    /// Checks to see if the current Version is in pre-release status
+    pub fn is_prerelease(&mut self) -> bool {
+        !self.pre.is_empty()
+    }
 }
 
 
@@ -161,17 +201,29 @@ impl cmp::Ord for Version {
     }
 }
 
+impl Error for ParseError {
+    fn description(&self) -> &str {
+        match *self {
+            ParseError::NonAsciiIdentifier
+                => "identifiers can only contain ascii characters",
+            ParseError::GenericFailure
+                | ParseError::IncorrectParse(..)
+                => "failed to parse semver from string",
+        }
+    }
+}
+
 impl fmt::Display for ParseError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
             ParseError::NonAsciiIdentifier => {
-                write!(f, "identifiers can only contain ascii characters")
+                write!(f, "{}", self.description())
             }
             ParseError::GenericFailure => {
-                write!(f, "failed to parse semver from string")
+                write!(f, "{}", self.description())
             }
             ParseError::IncorrectParse(ref a, ref b) => {
-                write!(f, "semver `{}` was not correctly parsed from {:?}", a, b)
+                write!(f, "{}: {} could not be parsed from {:?}", self.description(), a, b)
             }
         }
     }
@@ -391,6 +443,62 @@ mod test {
 
     }
 
+    #[test]
+    fn test_increment_patch() {
+        let mut buggy_release = Version::parse("0.1.0").unwrap();
+        buggy_release.increment_patch();
+        assert_eq!(buggy_release, Version::parse("0.1.1").unwrap());
+    }
+
+    #[test]
+    fn test_increment_minor() {
+        let mut feature_release = Version::parse("1.4.6").unwrap();
+        feature_release.increment_minor();
+        assert_eq!(feature_release, Version::parse("1.5.0").unwrap());
+    }
+
+    #[test]
+    fn test_increment_major() {
+        let mut chrome_release = Version::parse("46.1.246773").unwrap();
+        chrome_release.increment_major();
+        assert_eq!(chrome_release, Version::parse("47.0.0").unwrap());
+    }
+
+    #[test]
+    fn test_increment_keep_prerelease() {
+        let mut release = Version::parse("1.0.0-alpha").unwrap();
+        release.increment_patch();
+
+        assert_eq!(release, Version::parse("1.0.1").unwrap());
+
+        release.increment_minor();
+
+        assert_eq!(release, Version::parse("1.1.0").unwrap());
+
+        release.increment_major();
+
+        assert_eq!(release, Version::parse("2.0.0").unwrap());
+    }
+
+
+    #[test]
+    fn test_increment_clear_metadata() {
+        let mut release = Version::parse("1.0.0+4442").unwrap();
+        release.increment_patch();
+
+        assert_eq!(release, Version::parse("1.0.1").unwrap());
+        release = Version::parse("1.0.1+hello").unwrap();
+
+        release.increment_minor();
+
+        assert_eq!(release, Version::parse("1.1.0").unwrap());
+        release = Version::parse("1.1.3747+hello").unwrap();
+
+        release.increment_major();
+
+        assert_eq!(release, Version::parse("2.0.0").unwrap());
+    }
+
     #[test]
     fn test_eq() {
         assert_eq!(Version::parse("1.2.3"), Version::parse("1.2.3"));
@@ -470,6 +578,14 @@ mod test {
         assert!(Version::parse("1.2.3+23")     >= Version::parse("1.2.3+42"));
     }
 
+    #[test]
+    fn test_prerelease_check() {
+        assert!(Version::parse("1.0.0").unwrap().is_prerelease() == false);
+        assert!(Version::parse("0.0.1").unwrap().is_prerelease() == false);
+        assert!(Version::parse("4.1.4-alpha").unwrap().is_prerelease());
+        assert!(Version::parse("1.0.0-beta294296").unwrap().is_prerelease());
+    }
+
     #[test]
     fn test_spec_order() {
         let vs = ["1.0.0-alpha",
