@@ -15,9 +15,7 @@ use self::States::*;
 use self::FormatState::*;
 use self::FormatOp::*;
 
-use std::ascii::AsciiExt;
 use std::iter::repeat;
-use std::mem::replace;
 
 #[derive(Clone, Copy, PartialEq)]
 enum States {
@@ -28,7 +26,7 @@ enum States {
     PushParam,
     CharConstant,
     CharClose,
-    IntConstant(i16),
+    IntConstant(i32),
     FormatPattern(Flags, FormatState),
     SeekIfElse(usize),
     SeekIfElsePercent(usize),
@@ -48,7 +46,7 @@ enum FormatState {
 #[derive(Clone)]
 pub enum Param {
     Words(String),
-    Number(i16)
+    Number(i32)
 }
 
 /// Container for static and dynamic variable arrays
@@ -124,129 +122,63 @@ pub fn expand(cap: &[u8], params: &[Param], vars: &mut Variables)
             Percent => {
                 match cur {
                     '%' => { output.push(c); state = Nothing },
-                    'c' => if stack.len() > 0 {
-                        match stack.pop().unwrap() {
-                            // if c is 0, use 0200 (128) for ncurses compatibility
-                            Number(c) => {
-                                output.push(if c == 0 {
-                                    128u8
-                                } else {
-                                    c as u8
-                                })
-                            }
-                            _       => return Err("a non-char was used with %c".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
+                    'c' => match stack.pop() {
+                        // if c is 0, use 0200 (128) for ncurses compatibility
+                        Some(Number(0)) => output.push(128u8),
+                        // Don't check bounds. ncurses just casts and truncates.
+                        Some(Number(c)) => output.push(c as u8),
+                        Some(_) => return Err("a non-char was used with %c".to_string()),
+                        None => return Err("stack is empty".to_string()),
+                    },
                     'p' => state = PushParam,
                     'P' => state = SetVar,
                     'g' => state = GetVar,
                     '\'' => state = CharConstant,
                     '{' => state = IntConstant(0),
-                    'l' => if stack.len() > 0 {
-                        match stack.pop().unwrap() {
-                            Words(s) => stack.push(Number(s.len() as i16)),
-                            _        => return Err("a non-str was used with %l".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
-                    '+' => if stack.len() > 1 {
-                        match (stack.pop().unwrap(), stack.pop().unwrap()) {
-                            (Number(y), Number(x)) => stack.push(Number(x + y)),
-                            _ => return Err("non-numbers on stack with +".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
-                    '-' => if stack.len() > 1 {
-                        match (stack.pop().unwrap(), stack.pop().unwrap()) {
-                            (Number(y), Number(x)) => stack.push(Number(x - y)),
-                            _ => return Err("non-numbers on stack with -".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
-                    '*' => if stack.len() > 1 {
-                        match (stack.pop().unwrap(), stack.pop().unwrap()) {
-                            (Number(y), Number(x)) => stack.push(Number(x * y)),
-                            _ => return Err("non-numbers on stack with *".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
-                    '/' => if stack.len() > 1 {
-                        match (stack.pop().unwrap(), stack.pop().unwrap()) {
-                            (Number(y), Number(x)) => stack.push(Number(x / y)),
-                            _ => return Err("non-numbers on stack with /".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
-                    'm' => if stack.len() > 1 {
-                        match (stack.pop().unwrap(), stack.pop().unwrap()) {
-                            (Number(y), Number(x)) => stack.push(Number(x % y)),
-                            _ => return Err("non-numbers on stack with %".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
-                    '&' => if stack.len() > 1 {
-                        match (stack.pop().unwrap(), stack.pop().unwrap()) {
-                            (Number(y), Number(x)) => stack.push(Number(x & y)),
-                            _ => return Err("non-numbers on stack with &".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
-                    '|' => if stack.len() > 1 {
-                        match (stack.pop().unwrap(), stack.pop().unwrap()) {
-                            (Number(y), Number(x)) => stack.push(Number(x | y)),
-                            _ => return Err("non-numbers on stack with |".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
-                    '^' => if stack.len() > 1 {
-                        match (stack.pop().unwrap(), stack.pop().unwrap()) {
-                            (Number(y), Number(x)) => stack.push(Number(x ^ y)),
-                            _ => return Err("non-numbers on stack with ^".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
-                    '=' => if stack.len() > 1 {
-                        match (stack.pop().unwrap(), stack.pop().unwrap()) {
-                            (Number(y), Number(x)) => stack.push(Number(if x == y { 1 }
-                                                                        else { 0 })),
-                            _ => return Err("non-numbers on stack with =".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
-                    '>' => if stack.len() > 1 {
-                        match (stack.pop().unwrap(), stack.pop().unwrap()) {
-                            (Number(y), Number(x)) => stack.push(Number(if x > y { 1 }
-                                                                        else { 0 })),
-                            _ => return Err("non-numbers on stack with >".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
-                    '<' => if stack.len() > 1 {
-                        match (stack.pop().unwrap(), stack.pop().unwrap()) {
-                            (Number(y), Number(x)) => stack.push(Number(if x < y { 1 }
-                                                                        else { 0 })),
-                            _ => return Err("non-numbers on stack with <".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
-                    'A' => if stack.len() > 1 {
-                        match (stack.pop().unwrap(), stack.pop().unwrap()) {
-                            (Number(0), Number(_)) => stack.push(Number(0)),
-                            (Number(_), Number(0)) => stack.push(Number(0)),
-                            (Number(_), Number(_)) => stack.push(Number(1)),
-                            _ => return Err("non-numbers on stack with logical and".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
-                    'O' => if stack.len() > 1 {
-                        match (stack.pop().unwrap(), stack.pop().unwrap()) {
-                            (Number(0), Number(0)) => stack.push(Number(0)),
-                            (Number(_), Number(_)) => stack.push(Number(1)),
-                            _ => return Err("non-numbers on stack with logical or".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
-                    '!' => if stack.len() > 0 {
-                        match stack.pop().unwrap() {
-                            Number(0) => stack.push(Number(1)),
-                            Number(_) => stack.push(Number(0)),
-                            _ => return Err("non-number on stack with logical not".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
-                    '~' => if stack.len() > 0 {
-                        match stack.pop().unwrap() {
-                            Number(x) => stack.push(Number(!x)),
-                            _         => return Err("non-number on stack with %~".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
-                    'i' => match (mparams[0].clone(), mparams[1].clone()) {
-                        (Number(x), Number(y)) => {
+                    'l' => match stack.pop() {
+                        Some(Words(s)) => stack.push(Number(s.len() as i32)),
+                        Some(_) => return Err("a non-str was used with %l".to_string()),
+                        None => return Err("stack is empty".to_string()) 
+                    },
+                    '+'|'-'|'/'|'*'|'^'|'&'|'|'|'m' => match (stack.pop(), stack.pop()) {
+                        (Some(Number(y)), Some(Number(x))) => stack.push(Number(match cur {
+                            '+' => x + y,
+                            '-' => x - y,
+                            '*' => x * y,
+                            '/' => x / y,
+                            '|' => x | y,
+                            '&' => x & y,
+                            '^' => x ^ y,
+                            'm' => x % y,
+                            _ => unreachable!("All cases handled"),
+                        })),
+                        (Some(_), Some(_)) => return Err(format!("non-numbers on stack with {}", cur)),
+                        _ => return Err("stack is empty".to_string()),
+                    },
+                    '='|'>'|'<'|'A'|'O' => match (stack.pop(), stack.pop()) {
+                        (Some(Number(y)), Some(Number(x))) => stack.push(Number(if match cur {
+                            '=' => x == y,
+                            '<' => x < y,
+                            '>' => x > y,
+                            'A' => x > 0 && y > 0,
+                            'O' => x > 0 || y > 0,
+                            _ => unreachable!(),
+                        } { 1 } else { 0 })),
+                        (Some(_), Some(_)) => return Err(format!("non-numbers on stack with {}", cur)),
+                        _ => return Err("stack is empty".to_string()),
+                    },
+                    '!'|'~' => match stack.pop() {
+                        Some(Number(x)) => stack.push(Number(match cur {
+                            '!' if x > 0 => 0,
+                            '!' => 1,
+                            '~' => !x,
+                            _ => unreachable!(),
+                        })),
+                        Some(_) => return Err(format!("non-numbers on stack with {}", cur)),
+                        None => return Err("stack is empty".to_string()),
+                    },
+                    'i' => match (&mparams[0], &mparams[1]) {
+                        (&Number(x), &Number(y)) => {
                             mparams[0] = Number(x+1);
                             mparams[1] = Number(y+1);
                         },
@@ -254,11 +186,10 @@ pub fn expand(cap: &[u8], params: &[Param], vars: &mut Variables)
                     },
 
                     // printf-style support for %doxXs
-                    'd'|'o'|'x'|'X'|'s' => if stack.len() > 0 {
+                    'd'|'o'|'x'|'X'|'s' => if let Some(arg) = stack.pop() {
                         let flags = Flags::new();
-                        let res = format(stack.pop().unwrap(), FormatOp::from_char(cur), flags);
-                        if res.is_err() { return res }
-                        output.extend(res.unwrap().iter().map(|x| *x))
+                        let res = try!(format(arg, FormatOp::from_char(cur), flags));
+                        output.extend(res.iter().map(|x| *x));
                     } else { return Err("stack is empty".to_string()) },
                     ':'|'#'|' '|'.'|'0'...'9' => {
                         let mut flags = Flags::new();
@@ -279,20 +210,15 @@ pub fn expand(cap: &[u8], params: &[Param], vars: &mut Variables)
 
                     // conditionals
                     '?' => (),
-                    't' => if stack.len() > 0 {
-                        match stack.pop().unwrap() {
-                            Number(0) => state = SeekIfElse(0),
-                            Number(_) => (),
-                            _         => return Err("non-number on stack \
-                                                    with conditional".to_string())
-                        }
-                    } else { return Err("stack is empty".to_string()) },
+                    't' => match stack.pop() {
+                        Some(Number(0)) => state = SeekIfElse(0),
+                        Some(Number(_)) => (),
+                        Some(_) => return Err("non-number on stack with conditional".to_string()),
+                        None => return Err("stack is empty".to_string()),
+                    },
                     'e' => state = SeekIfEnd(0),
                     ';' => (),
-
-                    _ => {
-                        return Err(format!("unrecognized format option {}", cur))
-                    }
+                    _ => return Err(format!("unrecognized format option {}", cur)),
                 }
             },
             PushParam => {
@@ -304,14 +230,14 @@ pub fn expand(cap: &[u8], params: &[Param], vars: &mut Variables)
             },
             SetVar => {
                 if cur >= 'A' && cur <= 'Z' {
-                    if stack.len() > 0 {
+                    if let Some(arg) = stack.pop() {
                         let idx = (cur as u8) - b'A';
-                        vars.sta[idx as usize] = stack.pop().unwrap();
+                        vars.sta[idx as usize] = arg;
                     } else { return Err("stack is empty".to_string()) }
                 } else if cur >= 'a' && cur <= 'z' {
-                    if stack.len() > 0 {
+                    if let Some(arg) = stack.pop() {
                         let idx = (cur as u8) - b'a';
-                        vars.dyn[idx as usize] = stack.pop().unwrap();
+                        vars.dyn[idx as usize] = arg;
                     } else { return Err("stack is empty".to_string()) }
                 } else {
                     return Err("bad variable name in %P".to_string());
@@ -329,20 +255,18 @@ pub fn expand(cap: &[u8], params: &[Param], vars: &mut Variables)
                 }
             },
             CharConstant => {
-                stack.push(Number(c as i16));
+                stack.push(Number(c as i32));
                 state = CharClose;
             },
-            CharClose => {
-                if cur != '\'' {
-                    return Err("malformed character constant".to_string());
-                }
+            CharClose => if cur != '\'' {
+                return Err("malformed character constant".to_string());
             },
             IntConstant(i) => {
                 if cur == '}' {
                     stack.push(Number(i));
                     state = Nothing;
                 } else if let Some(digit) = cur.to_digit(10) {
-                    match i.checked_mul(10).and_then(|i_ten|i_ten.checked_add(digit as i16)) {
+                    match i.checked_mul(10).and_then(|i_ten|i_ten.checked_add(digit as i32)) {
                         Some(i) => {
                             state = IntConstant(i);
                             old_state = Nothing;
@@ -356,10 +280,9 @@ pub fn expand(cap: &[u8], params: &[Param], vars: &mut Variables)
             FormatPattern(ref mut flags, ref mut fstate) => {
                 old_state = Nothing;
                 match (*fstate, cur) {
-                    (_,'d')|(_,'o')|(_,'x')|(_,'X')|(_,'s') => if stack.len() > 0 {
-                        let res = format(stack.pop().unwrap(), FormatOp::from_char(cur), *flags);
-                        if res.is_err() { return res }
-                        output.extend(res.unwrap().iter().map(|x| *x));
+                    (_,'d')|(_,'o')|(_,'x')|(_,'X')|(_,'s') => if let Some(arg) = stack.pop() {
+                        let res = try!(format(arg, FormatOp::from_char(cur), *flags));
+                        output.extend(res.iter().map(|x| *x));
                         // will cause state to go to Nothing
                         old_state = FormatPattern(*flags, *fstate);
                     } else { return Err("stack is empty".to_string()) },
@@ -498,67 +421,52 @@ impl FormatOp {
 
 fn format(val: Param, op: FormatOp, flags: Flags) -> Result<Vec<u8> ,String> {
     let mut s = match val {
-        Number(d) => {
-            let s = match (op, flags.sign) {
-                (FormatDigit, true)  => format!("{:+}", d),
-                (FormatDigit, false) => format!("{}", d),
-                (FormatOctal, _)     => format!("{:o}", d),
-                (FormatHex, _)       => format!("{:x}", d),
-                (FormatHEX, _)       => format!("{:X}", d),
-                (FormatString, _)    => return Err("non-number on stack with %s".to_string())
-            };
-
-            let mut s: Vec<u8> = s.into_bytes().into_iter().collect();
-            if flags.precision > s.len() {
-                let mut s_ = Vec::with_capacity(flags.precision);
-                let n = flags.precision - s.len();
-                s_.extend(repeat(b'0').take(n));
-                s_.extend(s.into_iter());
-                s = s_;
-            }
-            assert!(!s.is_empty(), "string conversion produced empty result");
-            match op {
-                FormatDigit => {
-                    if flags.space && !(s[0] == b'-' || s[0] == b'+' ) {
-                        s.insert(0, b' ');
-                    }
+        Number(d) => match op {
+            FormatDigit => {
+                if flags.sign {
+                    format!("{:+01$}", d, flags.precision)
+                } else if d < 0 {
+                    // C doesn't take sign into account in precision calculation.
+                    format!("{:01$}", d, flags.precision + 1)
+                } else if flags.space {
+                    format!(" {:01$}", d, flags.precision)
+                } else {
+                    format!("{:01$}", d, flags.precision)
                 }
-                FormatOctal => {
-                    if flags.alternate && s[0] != b'0' {
-                        s.insert(0, b'0');
-                    }
+            },
+            FormatOctal => {
+                if flags.alternate {
+                    // Leading octal zero counts against precision.
+                    format!("0{:01$o}", d, flags.precision.saturating_sub(1))
+                } else {
+                    format!("{:01$o}", d, flags.precision)
                 }
-                FormatHex => {
-                    if flags.alternate {
-                        let s_ = replace(&mut s, vec!(b'0', b'x'));
-                        s.extend(s_.into_iter());
-                    }
+            },
+            FormatHex => {
+                if flags.alternate && d != 0 {
+                    format!("0x{:01$x}", d, flags.precision)
+                } else {
+                    format!("{:01$x}", d, flags.precision)
                 }
-                FormatHEX => {
-                    s = s.to_ascii_uppercase();
-                    if flags.alternate {
-                        let s_ = replace(&mut s, vec!(b'0', b'X'));
-                        s.extend(s_.into_iter());
-                    }
+            },
+            FormatHEX => {
+                if flags.alternate && d != 0 {
+                    format!("0X{:01$X}", d, flags.precision)
+                } else {
+                    format!("{:01$X}", d, flags.precision)
                 }
-                FormatString => unreachable!()
-            }
-            s
-        }
-        Words(s) => {
-            match op {
-                FormatString => {
-                    let mut s = s.as_bytes().to_vec();
-                    if flags.precision > 0 && flags.precision < s.len() {
-                        s.truncate(flags.precision);
-                    }
-                    s
+            },
+            FormatString => return Err("non-number on stack with %s".to_string())
+        }.into_bytes(),
+        Words(s) => match op {
+            FormatString => {
+                let mut s = s.into_bytes();
+                if flags.precision > 0 && flags.precision < s.len() {
+                    s.truncate(flags.precision);
                 }
-                _ => {
-                    return Err(format!("non-string on stack with %{}",
-                                       op.to_char()))
-                }
-            }
+                s
+            },
+            _ => return Err(format!("non-string on stack with %{}", op.to_char()))
         }
     };
     if flags.width > s.len() {
