@@ -25,23 +25,29 @@
 //! ```rust
 //! extern crate time;
 //! ```
+//!
+//! This crate uses the same syntax for format strings as the [strftime()]
+//! (http://man7.org/linux/man-pages/man3/strftime.3.html) function from the C
+//! standard library.
 
-#![doc(html_logo_url = "http://www.rust-lang.org/logos/rust-logo-128x128-blk-v2.png",
-       html_favicon_url = "http://www.rust-lang.org/favicon.ico",
-       html_root_url = "http://doc.rust-lang.org/time/")]
+#![doc(html_logo_url = "https://www.rust-lang.org/logos/rust-logo-128x128-blk-v2.png",
+       html_favicon_url = "https://www.rust-lang.org/favicon.ico",
+       html_root_url = "https://doc.rust-lang.org/time/")]
 #![allow(trivial_numeric_casts)]
 #![cfg_attr(test, deny(warnings))]
 
-#[cfg(test)] #[macro_use] extern crate log;
+#[cfg(unix)] extern crate libc;
+#[cfg(windows)] extern crate kernel32;
+#[cfg(windows)] extern crate winapi;
+#[cfg(feature = "rustc-serialize")] extern crate rustc_serialize;
 
-extern crate libc;
-#[cfg(feature = "rustc-serialize")]
-extern crate rustc_serialize;
+#[cfg(test)] #[macro_use] extern crate log;
+#[cfg(all(windows, test))] extern crate advapi32;
 
 use std::cmp::Ordering;
+use std::error::Error;
 use std::fmt;
 use std::ops::{Add, Sub};
-use std::io;
 
 pub use duration::Duration;
 
@@ -54,89 +60,18 @@ use self::ParseError::{InvalidDay, InvalidDayOfMonth, InvalidDayOfWeek,
 pub use parse::strptime;
 
 mod display;
-mod parse;
 mod duration;
+mod parse;
+mod sys;
 
 static NSEC_PER_SEC: i32 = 1_000_000_000;
 
-mod rustrt {
-    use super::Tm;
-
-    extern {
-        pub fn rust_time_gmtime(sec: i64, nsec: i32, result: &mut Tm);
-        pub fn rust_time_localtime(sec: i64, nsec: i32, result: &mut Tm) -> i32;
-        pub fn rust_time_timegm(tm: &Tm) -> i64;
-        pub fn rust_time_mktime(tm: &Tm) -> i64;
-    }
-}
-
-#[cfg(all(unix, not(target_os = "macos"), not(target_os = "ios")))]
-mod imp {
-    use libc::{c_int, timespec};
-
-    #[cfg(all(not(target_os = "android"),
-              not(target_os = "bitrig"),
-              not(target_os = "nacl"),
-              not(target_os = "openbsd")))]
-    #[link(name = "rt")]
-    extern {}
-
-    extern {
-        pub fn clock_gettime(clk_id: c_int, tp: *mut timespec) -> c_int;
-    }
-
-}
-#[cfg(any(target_os = "macos", target_os = "ios"))]
-mod imp {
-    use libc::{timeval, timezone, c_int, mach_timebase_info};
-    use std::sync::{Once, ONCE_INIT};
-
-    extern {
-        pub fn gettimeofday(tp: *mut timeval, tzp: *mut timezone) -> c_int;
-        pub fn mach_absolute_time() -> u64;
-        pub fn mach_timebase_info(info: *mut mach_timebase_info) -> c_int;
-    }
-
-    pub fn info() -> &'static mach_timebase_info {
-        static mut INFO: mach_timebase_info = mach_timebase_info {
-            numer: 0,
-            denom: 0,
-        };
-        static ONCE: Once = ONCE_INIT;
-
-        unsafe {
-            ONCE.call_once(|| {
-                mach_timebase_info(&mut INFO);
-            });
-            &INFO
-        }
-    }
-}
-
-#[cfg(windows)]
-mod imp {
-    use libc;
-    use std::sync::{Once, ONCE_INIT};
-
-    pub fn frequency() -> libc::LARGE_INTEGER {
-        static mut FREQUENCY: libc::LARGE_INTEGER = 0;
-        static ONCE: Once = ONCE_INIT;
-
-        unsafe {
-            ONCE.call_once(|| {
-                libc::QueryPerformanceFrequency(&mut FREQUENCY);
-            });
-            FREQUENCY
-        }
-    }
-}
-
 /// A record specifying a time value in seconds and nanoseconds, where
 /// nanoseconds represent the offset from the given second.
 ///
 /// For example a timespec of 1.2 seconds after the beginning of the epoch would
 /// be represented as {sec: 1, nsec: 200000000}.
-#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Debug)]
+#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Debug, Hash)]
 #[cfg_attr(feature = "rustc-serialize", derive(RustcEncodable, RustcDecodable))]
 pub struct Timespec { pub sec: i64, pub nsec: i32 }
 /*
@@ -213,46 +148,8 @@ impl Sub<Timespec> for Timespec {
  * nanoseconds since 1970-01-01T00:00:00Z.
  */
 pub fn get_time() -> Timespec {
-    unsafe {
-        let (sec, nsec) = os_get_time();
-        return Timespec::new(sec, nsec);
-    }
-
-    #[cfg(windows)]
-    unsafe fn os_get_time() -> (i64, i32) {
-        static MICROSECONDS_FROM_1601_TO_1970: u64 = 11644473600000000;
-
-        let mut time = libc::FILETIME {
-            dwLowDateTime: 0,
-            dwHighDateTime: 0,
-        };
-        libc::GetSystemTimeAsFileTime(&mut time);
-
-        // A FILETIME contains a 64-bit value representing the number of
-        // hectonanosecond (100-nanosecond) intervals since 1601-01-01T00:00:00Z.
-        // http://support.microsoft.com/kb/167296/en-us
-        let us_since_1601 = (((time.dwHighDateTime as u64) << 32) |
-                             ((time.dwLowDateTime  as u64) <<  0)) / 10;
-        let us_since_1970 = us_since_1601 - MICROSECONDS_FROM_1601_TO_1970;
-
-        ((us_since_1970 / 1000000) as i64,
-         ((us_since_1970 % 1000000) * 1000) as i32)
-    }
-
-    #[cfg(any(target_os = "macos", target_os = "ios"))]
-    unsafe fn os_get_time() -> (i64, i32) {
-        use std::ptr;
-        let mut tv = libc::timeval { tv_sec: 0, tv_usec: 0 };
-        imp::gettimeofday(&mut tv, ptr::null_mut());
-        (tv.tv_sec as i64, tv.tv_usec * 1000)
-    }
-
-    #[cfg(not(any(target_os = "macos", target_os = "ios", windows)))]
-    unsafe fn os_get_time() -> (i64, i32) {
-        let mut tv = libc::timespec { tv_sec: 0, tv_nsec: 0 };
-        imp::clock_gettime(libc::CLOCK_REALTIME, &mut tv);
-        (tv.tv_sec as i64, tv.tv_nsec as i32)
-    }
+    let (sec, nsec) = sys::get_time();
+    Timespec::new(sec, nsec)
 }
 
 
@@ -261,35 +158,7 @@ pub fn get_time() -> Timespec {
  * in nanoseconds since an unspecified epoch.
  */
 pub fn precise_time_ns() -> u64 {
-    return os_precise_time_ns();
-
-    #[cfg(windows)]
-    fn os_precise_time_ns() -> u64 {
-        let mut ticks = 0;
-        assert_eq!(unsafe {
-            libc::QueryPerformanceCounter(&mut ticks)
-        }, 1);
-
-        mul_div_i64(ticks as i64, 1000000000, imp::frequency() as i64) as u64
-    }
-
-    #[cfg(any(target_os = "macos", target_os = "ios"))]
-    fn os_precise_time_ns() -> u64 {
-        unsafe {
-            let time = imp::mach_absolute_time();
-            let info = imp::info();
-            time * info.numer as u64 / info.denom as u64
-        }
-    }
-
-    #[cfg(not(any(windows, target_os = "macos", target_os = "ios")))]
-    fn os_precise_time_ns() -> u64 {
-        let mut ts = libc::timespec { tv_sec: 0, tv_nsec: 0 };
-        unsafe {
-            imp::clock_gettime(libc::CLOCK_MONOTONIC, &mut ts);
-        }
-        return (ts.tv_sec as u64) * 1000000000 + (ts.tv_nsec as u64)
-    }
+    sys::get_precise_ns()
 }
 
 
@@ -371,12 +240,12 @@ impl PreciseTime {
 /// }
 /// ```
 #[derive(Clone, Copy, PartialOrd, Ord, PartialEq, Eq, Debug)]
-pub struct SteadyTime(steady::SteadyTime);
+pub struct SteadyTime(sys::SteadyTime);
 
 impl SteadyTime {
     /// Returns a `SteadyTime` representing the current moment in time.
     pub fn now() -> SteadyTime {
-        SteadyTime(steady::SteadyTime::now())
+        SteadyTime(sys::SteadyTime::now())
     }
 }
 
@@ -411,222 +280,22 @@ impl Add<Duration> for SteadyTime {
     }
 }
 
-#[cfg(any(target_os = "macos", target_os = "ios"))]
-mod steady {
-    use imp;
-    use Duration;
-    use std::ops::{Sub, Add};
-
-    #[derive(Clone, Copy, PartialOrd, Ord, PartialEq, Eq, Debug)]
-    pub struct SteadyTime {
-        t: u64,
-    }
-
-    impl SteadyTime {
-        pub fn now() -> SteadyTime {
-            SteadyTime {
-                t: unsafe { imp::mach_absolute_time() },
-            }
-        }
-    }
-
-    impl Sub for SteadyTime {
-        type Output = Duration;
-
-        fn sub(self, other: SteadyTime) -> Duration {
-            let info = imp::info();
-            let diff = self.t as i64 - other.t as i64;
-            Duration::nanoseconds(diff * info.numer as i64 / info.denom as i64)
-        }
-    }
-
-    impl Sub<Duration> for SteadyTime {
-        type Output = SteadyTime;
-
-        fn sub(self, other: Duration) -> SteadyTime {
-            self + -other
-        }
-    }
-
-    impl Add<Duration> for SteadyTime {
-        type Output = SteadyTime;
-
-        fn add(self, other: Duration) -> SteadyTime {
-            let info = imp::info();
-            let delta = other.num_nanoseconds().unwrap() * info.denom as i64 / info.numer as i64;
-            SteadyTime {
-                t: (self.t as i64 + delta) as u64
-            }
-        }
-    }
-}
-
-#[cfg(not(any(windows, target_os = "macos", target_os = "ios")))]
-mod steady {
-    use {imp, NSEC_PER_SEC, Duration};
-    use libc;
-    use std::cmp::{PartialOrd, Ord, Ordering, PartialEq, Eq};
-    use std::ops::{Sub, Add};
-    use std::fmt;
-
-    #[derive(Copy)]
-    pub struct SteadyTime {
-        t: libc::timespec,
-    }
-
-    impl fmt::Debug for SteadyTime {
-        fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
-            write!(fmt, "SteadyTime {{ tv_sec: {:?}, tv_nsec: {:?} }}",
-                   self.t.tv_sec, self.t.tv_nsec)
-        }
-    }
-
-    impl Clone for SteadyTime {
-        fn clone(&self) -> SteadyTime {
-            SteadyTime { t: self.t }
-        }
-    }
-
-    impl SteadyTime {
-        pub fn now() -> SteadyTime {
-            let mut t = SteadyTime {
-                t: libc::timespec {
-                    tv_sec: 0,
-                    tv_nsec: 0,
-                }
-            };
-            unsafe {
-                assert_eq!(0, imp::clock_gettime(libc::CLOCK_MONOTONIC, &mut t.t));
-            }
-            t
-        }
-    }
-
-    impl Sub for SteadyTime {
-        type Output = Duration;
-
-        fn sub(self, other: SteadyTime) -> Duration {
-            if self.t.tv_nsec >= other.t.tv_nsec {
-                Duration::seconds(self.t.tv_sec as i64 - other.t.tv_sec as i64) +
-                    Duration::nanoseconds(self.t.tv_nsec as i64 - other.t.tv_nsec as i64)
-            } else {
-                Duration::seconds(self.t.tv_sec as i64 - 1 - other.t.tv_sec as i64) +
-                    Duration::nanoseconds(self.t.tv_nsec as i64 + NSEC_PER_SEC as i64 -
-                                          other.t.tv_nsec as i64)
-            }
-        }
-    }
-
-    impl Sub<Duration> for SteadyTime {
-        type Output = SteadyTime;
-
-        fn sub(self, other: Duration) -> SteadyTime {
-            self + -other
-        }
-    }
-
-    impl Add<Duration> for SteadyTime {
-        type Output = SteadyTime;
-
-        fn add(mut self, other: Duration) -> SteadyTime {
-            let seconds = other.num_seconds();
-            let nanoseconds = (other - Duration::seconds(seconds)).num_nanoseconds().unwrap();
-            self.t.tv_sec += seconds as libc::time_t;
-            self.t.tv_nsec += nanoseconds as libc::c_long;
-            if self.t.tv_nsec >= NSEC_PER_SEC as libc::c_long {
-                self.t.tv_nsec -= NSEC_PER_SEC as libc::c_long;
-                self.t.tv_sec += 1;
-            } else if self.t.tv_nsec < 0 {
-                self.t.tv_sec -= 1;
-                self.t.tv_nsec += NSEC_PER_SEC as libc::c_long;
-            }
-            self
-        }
-    }
-
-    impl PartialOrd for SteadyTime {
-        fn partial_cmp(&self, other: &SteadyTime) -> Option<Ordering> {
-            Some(self.cmp(other))
-        }
-    }
-
-    impl Ord for SteadyTime {
-        fn cmp(&self, other: &SteadyTime) -> Ordering {
-            match self.t.tv_sec.cmp(&other.t.tv_sec) {
-                Ordering::Equal => self.t.tv_nsec.cmp(&other.t.tv_nsec),
-                ord => ord
-            }
-        }
-    }
-
-    impl PartialEq for SteadyTime {
-        fn eq(&self, other: &SteadyTime) -> bool {
-            self.t.tv_sec == other.t.tv_sec && self.t.tv_nsec == other.t.tv_nsec
-        }
-    }
-
-    impl Eq for SteadyTime {}
-}
-
-#[cfg(windows)]
-mod steady {
-    use {imp, Duration};
-    use libc;
-    use std::ops::{Sub, Add};
-
-    #[derive(Clone, Copy, PartialOrd, Ord, PartialEq, Eq, Debug)]
-    pub struct SteadyTime {
-        t: libc::LARGE_INTEGER,
-    }
-
-    impl SteadyTime {
-        pub fn now() -> SteadyTime {
-            let mut t = SteadyTime { t: 0 };
-            unsafe { libc::QueryPerformanceCounter(&mut t.t); }
-            t
-        }
-    }
-
-    impl Sub for SteadyTime {
-        type Output = Duration;
-
-        fn sub(self, other: SteadyTime) -> Duration {
-            let diff = self.t as i64 - other.t as i64;
-            Duration::nanoseconds(::mul_div_i64(diff, 1000000000,
-                                                imp::frequency() as i64))
-        }
-    }
-
-    impl Sub<Duration> for SteadyTime {
-        type Output = SteadyTime;
-
-        fn sub(self, other: Duration) -> SteadyTime {
-            self + -other
-        }
-    }
-
-    impl Add<Duration> for SteadyTime {
-        type Output = SteadyTime;
-
-        fn add(mut self, other: Duration) -> SteadyTime {
-            self.t += (other.num_microseconds().unwrap() * imp::frequency() as i64 / 1_000_000)
-                as libc::LARGE_INTEGER;
-            self
-        }
-    }
-}
-
+#[cfg(not(windows))]
 pub fn tzset() {
-    #[cfg(windows)] extern { #[link_name = "_tzset"] fn tzset(); }
-    #[cfg(not(windows))] extern { fn tzset(); }
+    extern { fn tzset(); }
     unsafe { tzset() }
 }
 
+
+#[cfg(windows)]
+pub fn tzset() {}
+
 /// Holds a calendar date and time broken down into its components (year, month,
 /// day, and so on), also called a broken-down time value.
 // FIXME: use c_int instead of i32?
 #[repr(C)]
-#[derive(Copy, Clone, PartialEq, Eq, Debug)]
+#[derive(Copy, Clone, PartialEq, Eq, Debug, Hash)]
+#[cfg_attr(feature = "rustc-serialize", derive(RustcEncodable, RustcDecodable))]
 pub struct Tm {
     /// Seconds after the minute - [0, 60]
     pub tm_sec: i32,
@@ -731,12 +400,11 @@ pub fn empty_tm() -> Tm {
 
 /// Returns the specified time in UTC
 pub fn at_utc(clock: Timespec) -> Tm {
-    unsafe {
-        let Timespec { sec, nsec } = clock;
-        let mut tm = empty_tm();
-        rustrt::rust_time_gmtime(sec, nsec, &mut tm);
-        tm
-    }
+    let Timespec { sec, nsec } = clock;
+    let mut tm = empty_tm();
+    sys::time_to_utc_tm(sec, &mut tm);
+    tm.tm_nsec = nsec;
+    tm
 }
 
 /// Returns the current time in UTC
@@ -746,15 +414,11 @@ pub fn now_utc() -> Tm {
 
 /// Returns the specified time in the local timezone
 pub fn at(clock: Timespec) -> Tm {
-    unsafe {
-        let Timespec { sec, nsec } = clock;
-        let mut tm = empty_tm();
-        if rustrt::rust_time_localtime(sec, nsec, &mut tm) == 0 {
-            panic!("failed to call localtime: {}",
-                   io::Error::last_os_error());
-        }
-        tm
-    }
+    let Timespec { sec, nsec } = clock;
+    let mut tm = empty_tm();
+    sys::time_to_local_tm(sec, &mut tm);
+    tm.tm_nsec = nsec;
+    tm
 }
 
 /// Returns the current time in the local timezone
@@ -765,14 +429,12 @@ pub fn now() -> Tm {
 impl Tm {
     /// Convert time to the seconds from January 1, 1970
     pub fn to_timespec(&self) -> Timespec {
-        unsafe {
-            let sec = match self.tm_utcoff {
-                0 => rustrt::rust_time_timegm(self),
-                _     => rustrt::rust_time_mktime(self)
-            };
+        let sec = match self.tm_utcoff {
+            0 => sys::utc_tm_to_time(self),
+            _ => sys::local_tm_to_time(self)
+        };
 
-            Timespec::new(sec, self.tm_nsec)
-        }
+        Timespec::new(sec, self.tm_nsec)
     }
 
     /// Convert time to the local timezone
@@ -890,27 +552,35 @@ pub enum ParseError {
 impl fmt::Display for ParseError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
-            InvalidSecond => write!(f, "Invalid second."),
-            InvalidMinute => write!(f, "Invalid minute."),
-            InvalidHour => write!(f, "Invalid hour."),
-            InvalidDay => write!(f, "Invalid day."),
-            InvalidMonth => write!(f, "Invalid month."),
-            InvalidYear => write!(f, "Invalid year."),
-            InvalidDayOfWeek => write!(f, "Invalid day of the week."),
-            InvalidDayOfMonth => write!(f, "Invalid day of the month."),
-            InvalidDayOfYear => write!(f, "Invalid day of the year."),
-            InvalidZoneOffset => write!(f, "Invalid zone offset."),
-            InvalidTime => write!(f, "Invalid time."),
-            InvalidSecondsSinceEpoch => write!(f, "Invalid seconds since epoch."),
-            MissingFormatConverter => {
-                write!(f, "missing format converter after `%`")
-            }
             InvalidFormatSpecifier(ch) => {
-                write!(f, "invalid format specifier: %{}", ch)
+                write!(f, "{}: %{}", self.description(), ch)
             }
             UnexpectedCharacter(a, b) => {
                 write!(f, "expected: `{}`, found: `{}`", a, b)
             }
+            _ => write!(f, "{}", self.description())
+        }
+    }
+}
+
+impl Error for ParseError {
+    fn description(&self) -> &str {
+        match *self {
+            InvalidSecond => "Invalid second.",
+            InvalidMinute => "Invalid minute.",
+            InvalidHour => "Invalid hour.",
+            InvalidDay => "Invalid day.",
+            InvalidMonth => "Invalid month.",
+            InvalidYear => "Invalid year.",
+            InvalidDayOfWeek => "Invalid day of the week.",
+            InvalidDayOfMonth => "Invalid day of the month.",
+            InvalidDayOfYear => "Invalid day of the year.",
+            InvalidZoneOffset => "Invalid zone offset.",
+            InvalidTime => "Invalid time.",
+            InvalidSecondsSinceEpoch => "Invalid seconds since epoch.",
+            MissingFormatConverter => "missing format converter after `%`",
+            InvalidFormatSpecifier(..) => "invalid format specifier",
+            UnexpectedCharacter(..) => "Unexpected character.",
         }
     }
 }
@@ -975,69 +645,54 @@ pub fn strftime(format: &str, tm: &Tm) -> Result<String, ParseError> {
     tm.strftime(format).map(|fmt| fmt.to_string())
 }
 
-// Computes (value*numer)/denom without overflow, as long as both
-// (numer*denom) and the overall result fit into i64 (which is the case
-// for our time conversions).
-#[allow(dead_code)]
-fn mul_div_i64(value: i64, numer: i64, denom: i64) -> i64 {
-    let q = value / denom;
-    let r = value % denom;
-    // Decompose value as (value/denom*denom + value%denom),
-    // substitute into (value*numer)/denom and simplify.
-    // r < denom, so (denom*numer) is the upper bound of (r*numer)
-    q * numer + r * numer / denom
-}
-
 #[cfg(test)]
 mod tests {
-    use super::{Timespec, get_time, precise_time_ns, precise_time_s, tzset,
-                at_utc, at, strptime, PreciseTime, ParseError, Duration};
-    use super::mul_div_i64;
+    use super::{Timespec, get_time, precise_time_ns, precise_time_s,
+                at_utc, at, strptime, PreciseTime, SteadyTime, ParseError, Duration};
     use super::ParseError::{InvalidTime, InvalidYear, MissingFormatConverter,
                             InvalidFormatSpecifier};
 
+    use std::sync::{Once, ONCE_INIT, Mutex, MutexGuard, LockResult};
+    use std::mem;
 
-    #[test]
-    fn test_muldiv() {
-        assert_eq!(mul_div_i64( 1_000_000_000_001, 1_000_000_000, 1_000_000),
-                   1_000_000_000_001_000);
-        assert_eq!(mul_div_i64(-1_000_000_000_001, 1_000_000_000, 1_000_000),
-                   -1_000_000_000_001_000);
-        assert_eq!(mul_div_i64(-1_000_000_000_001,-1_000_000_000, 1_000_000),
-                   1_000_000_000_001_000);
-        assert_eq!(mul_div_i64( 1_000_000_000_001, 1_000_000_000,-1_000_000),
-                   -1_000_000_000_001_000);
-        assert_eq!(mul_div_i64( 1_000_000_000_001,-1_000_000_000,-1_000_000),
-                   1_000_000_000_001_000);
+    struct TzReset {
+        _tzreset: ::sys::TzReset,
+        _lock: LockResult<MutexGuard<'static, ()>>,
     }
 
-    #[cfg(windows)]
-    fn set_time_zone() {
-        use libc;
-        use std::ffi::CString;
-        // Windows crt doesn't see any environment variable set by
-        // `SetEnvironmentVariable`, which `os::setenv` internally uses.
-        // It is why we use `putenv` here.
-        extern {
-            fn _putenv(envstring: *const libc::c_char) -> libc::c_int;
-        }
+    fn set_time_zone_la_or_london(london: bool) -> TzReset {
+        // Lock manages current timezone because some tests require LA some
+        // London
+        static mut LOCK: *mut Mutex<()> = 0 as *mut _;
+        static INIT: Once = ONCE_INIT;
 
         unsafe {
-            // Windows does not understand "America/Los_Angeles".
-            // PST+08 may look wrong, but not! "PST" indicates
-            // the name of timezone. "+08" means UTC = local + 08.
-            let c = CString::new("TZ=PST+08").unwrap();
-            _putenv(c.as_ptr());
+            INIT.call_once(|| {
+                LOCK = mem::transmute(Box::new(Mutex::new(())));
+            });
+
+            let timezone_lock = (*LOCK).lock();
+            let reset_func = if london {
+                ::sys::set_london_with_dst_time_zone()
+            } else {
+                ::sys::set_los_angeles_time_zone()
+            };
+            TzReset {
+                _lock: timezone_lock,
+                _tzreset: reset_func,
+            }
         }
-        tzset();
-    }
-    #[cfg(not(windows))]
-    fn set_time_zone() {
-        use std::env;
-        env::set_var("TZ", "America/Los_Angeles");
-        tzset();
     }
 
+    fn set_time_zone() -> TzReset {
+        set_time_zone_la_or_london(false)
+    }
+
+    fn set_time_zone_london_dst() -> TzReset {
+        set_time_zone_la_or_london(true)
+    }
+
+    #[test]
     fn test_get_time() {
         static SOME_RECENT_DATE: i64 = 1325376000i64; // 2012-01-01T00:00:00Z
         static SOME_FUTURE_DATE: i64 = 1577836800i64; // 2020-01-01T00:00:00Z
@@ -1059,6 +714,7 @@ mod tests {
         }
     }
 
+    #[test]
     fn test_precise_time() {
         let s0 = precise_time_s();
         debug!("s0={} sec", s0);
@@ -1075,14 +731,16 @@ mod tests {
         assert!(ns2 >= ns1);
     }
 
+    #[test]
     fn test_precise_time_to() {
         let t0 = PreciseTime(1000);
         let t1 = PreciseTime(1023);
         assert_eq!(Duration::nanoseconds(23), t0.to(t1));
     }
 
+    #[test]
     fn test_at_utc() {
-        set_time_zone();
+        let _reset = set_time_zone();
 
         let time = Timespec::new(1234567890, 54321);
         let utc = at_utc(time);
@@ -1100,8 +758,9 @@ mod tests {
         assert_eq!(utc.tm_nsec, 54321);
     }
 
+    #[test]
     fn test_at() {
-        set_time_zone();
+        let _reset = set_time_zone();
 
         let time = Timespec::new(1234567890, 54321);
         let local = at(time);
@@ -1121,8 +780,9 @@ mod tests {
         assert_eq!(local.tm_nsec, 54321);
     }
 
+    #[test]
     fn test_to_timespec() {
-        set_time_zone();
+        let _reset = set_time_zone();
 
         let time = Timespec::new(1234567890, 54321);
         let utc = at_utc(time);
@@ -1131,8 +791,9 @@ mod tests {
         assert_eq!(utc.to_local().to_timespec(), time);
     }
 
+    #[test]
     fn test_conversions() {
-        set_time_zone();
+        let _reset = set_time_zone();
 
         let time = Timespec::new(1234567890, 54321);
         let utc = at_utc(time);
@@ -1146,8 +807,9 @@ mod tests {
         assert!(utc.to_local().to_utc() == utc);
     }
 
+    #[test]
     fn test_strptime() {
-        set_time_zone();
+        let _reset = set_time_zone();
 
         match strptime("", "") {
             Ok(ref tm) => {
@@ -1318,6 +980,7 @@ mod tests {
 
         assert!(strptime("-0000", "%z").unwrap().tm_utcoff == 0);
         assert!(strptime("-00:00", "%z").unwrap().tm_utcoff == 0);
+        assert!(strptime("Z", "%z").unwrap().tm_utcoff == 0);
         assert_eq!(-28800, strptime("-0800", "%z").unwrap().tm_utcoff);
         assert_eq!(-28800, strptime("-08:00", "%z").unwrap().tm_utcoff);
         assert_eq!(28800, strptime("+0800", "%z").unwrap().tm_utcoff);
@@ -1344,8 +1007,9 @@ mod tests {
         }
     }
 
+    #[test]
     fn test_asctime() {
-        set_time_zone();
+        let _reset = set_time_zone();
 
         let time = Timespec::new(1234567890, 54321);
         let utc   = at_utc(time);
@@ -1357,8 +1021,9 @@ mod tests {
         assert_eq!(local.asctime().to_string(), "Fri Feb 13 15:31:30 2009".to_string());
     }
 
+    #[test]
     fn test_ctime() {
-        set_time_zone();
+        let _reset = set_time_zone();
 
         let time = Timespec::new(1234567890, 54321);
         let utc   = at_utc(time);
@@ -1370,8 +1035,9 @@ mod tests {
         assert_eq!(local.ctime().to_string(), "Fri Feb 13 15:31:30 2009".to_string());
     }
 
+    #[test]
     fn test_strftime() {
-        set_time_zone();
+        let _reset = set_time_zone();
 
         let time = Timespec::new(1234567890, 54321);
         let utc = at_utc(time);
@@ -1448,6 +1114,7 @@ mod tests {
         assert_eq!(utc.rfc3339().to_string(), "2009-02-13T23:31:30Z".to_string());
     }
 
+    #[test]
     fn test_timespec_eq_ord() {
         let a = &Timespec::new(-2, 1);
         let b = &Timespec::new(-1, 2);
@@ -1479,6 +1146,40 @@ mod tests {
         assert!(d.gt(c));
     }
 
+    #[test]
+    fn test_timespec_hash() {
+        use std::hash::{Hash, Hasher};
+
+        let c = &Timespec::new(3, 2);
+        let d = &Timespec::new(2, 1);
+        let e = &Timespec::new(2, 1);
+
+        let mut hasher = ::std::hash::SipHasher::new();
+
+        let d_hash:u64 = {
+          d.hash(&mut hasher);
+          hasher.finish()
+        };
+
+        hasher = ::std::hash::SipHasher::new();
+
+        let e_hash:u64 = {
+          e.hash(&mut hasher);
+          hasher.finish()
+        };
+
+        hasher = ::std::hash::SipHasher::new();
+
+        let c_hash:u64 = {
+          c.hash(&mut hasher);
+          hasher.finish()
+        };
+
+        assert_eq!(d_hash, e_hash);
+        assert!(c_hash != e_hash);
+    }
+
+    #[test]
     fn test_timespec_add() {
         let a = Timespec::new(1, 2);
         let b = Duration::seconds(2) + Duration::nanoseconds(3);
@@ -1505,6 +1206,7 @@ mod tests {
         assert_eq!(m.nsec, 999_999_999);
     }
 
+    #[test]
     fn test_timespec_sub() {
         let a = Timespec::new(2, 3);
         let b = Timespec::new(1, 2);
@@ -1522,6 +1224,7 @@ mod tests {
         assert_eq!(w.num_nanoseconds(), Some(-super::NSEC_PER_SEC as i64 - 1));
     }
 
+    #[test]
     fn test_time_sub() {
         let a = ::now();
         let b = at(a.to_timespec() + Duration::seconds(5));
@@ -1530,24 +1233,43 @@ mod tests {
     }
 
     #[test]
-    #[cfg_attr(target_os = "android", ignore)] // FIXME #10958
-    fn run_tests() {
-        // The tests race on tzset. So instead of having many independent
-        // tests, we will just call the functions now.
-        test_get_time();
-        test_precise_time();
-        test_precise_time_to();
-        test_at_utc();
-        test_at();
-        test_to_timespec();
-        test_conversions();
-        test_strptime();
-        test_asctime();
-        test_ctime();
-        test_strftime();
-        test_timespec_eq_ord();
-        test_timespec_add();
-        test_timespec_sub();
-        test_time_sub();
+    fn test_steadytime_sub() {
+        let a = SteadyTime::now();
+        let b = a + Duration::seconds(1);
+        assert_eq!(b - a, Duration::seconds(1));
+        assert_eq!(a - b, Duration::seconds(-1));
+    }
+
+    #[test]
+    fn test_date_before_1970() {
+        let early = strptime("1901-01-06", "%F").unwrap();
+        let late = strptime("2000-01-01", "%F").unwrap();
+        assert!(early < late);
+    }
+
+    #[test]
+    fn test_dst() {
+        let _reset = set_time_zone_london_dst();
+        let utc_in_feb = strptime("2015-02-01Z", "%F%z").unwrap();
+        let utc_in_jun = strptime("2015-06-01Z", "%F%z").unwrap();
+        let utc_in_nov = strptime("2015-11-01Z", "%F%z").unwrap();
+        let local_in_feb = utc_in_feb.to_local();
+        let local_in_jun = utc_in_jun.to_local();
+        let local_in_nov = utc_in_nov.to_local();
+
+        assert_eq!(local_in_feb.tm_mon, 1);
+        assert_eq!(local_in_feb.tm_hour, 0);
+        assert_eq!(local_in_feb.tm_utcoff, 0);
+        assert_eq!(local_in_feb.tm_isdst, 0);
+
+        assert_eq!(local_in_jun.tm_mon, 5);
+        assert_eq!(local_in_jun.tm_hour, 1);
+        assert_eq!(local_in_jun.tm_utcoff, 3600);
+        assert_eq!(local_in_jun.tm_isdst, 1);
+
+        assert_eq!(local_in_nov.tm_mon, 10);
+        assert_eq!(local_in_nov.tm_hour, 0);
+        assert_eq!(local_in_nov.tm_utcoff, 0);
+        assert_eq!(local_in_nov.tm_isdst, 0)
     }
 }
