@@ -103,7 +103,7 @@ pub struct Parser<'a> {
 ///
 /// The data in this structure can be used to trace back to the original cause
 /// of the error in order to provide diagnostics about parse errors.
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct ParserError {
     /// The low byte at which this error is pointing at.
     pub lo: usize,
@@ -148,7 +148,7 @@ impl<'a> Parser<'a> {
     pub fn to_linecol(&self, offset: usize) -> (usize, usize) {
         let mut cur = 0;
         for (i, line) in self.input.lines().enumerate() {
-            if cur + line.len() > offset {
+            if cur + line.len() + 1 > offset {
                 return (i, offset - cur)
             }
             cur += line.len() + 1;
@@ -190,6 +190,14 @@ impl<'a> Parser<'a> {
         false
     }
 
+    // Consumes a BOM (Byte Order Mark) if one is next
+    fn bom(&mut self) -> bool {
+        match self.peek(0) {
+            Some((_, '\u{feff}')) => { self.cur.next(); true }
+            _ => false
+        }
+    }
+
     // Consumes whitespace ('\t' and ' ') until another character (or EOF) is
     // reached. Returns if any whitespace was consumed
     fn ws(&mut self) -> bool {
@@ -234,6 +242,7 @@ impl<'a> Parser<'a> {
     /// to determine the cause of the parse failure.
     pub fn parse(&mut self) -> Option<super::Table> {
         let mut ret = TomlTable { values: BTreeMap::new(), defined: false };
+        self.bom();
         while self.peek(0).is_some() {
             self.ws();
             if self.newline() { continue }
