@@ -2,16 +2,15 @@
 // Licensed under the MIT License <LICENSE.md>
 //! ApiSet Contract for api-ms-win-core-timezone-l1
 pub const TIME_ZONE_ID_INVALID: ::DWORD = 0xFFFFFFFF;
-#[repr(C)] #[derive(Clone, Copy, Debug)]
-pub struct TIME_ZONE_INFORMATION {
-    pub Bias: ::LONG,
-    pub StandardName: [::WCHAR; 32],
-    pub StandardDate: ::SYSTEMTIME,
-    pub StandardBias: ::LONG,
-    pub DaylightName: [::WCHAR; 32],
-    pub DaylightDate: ::SYSTEMTIME,
-    pub DaylightBias: ::LONG,
-}
+STRUCT!{struct TIME_ZONE_INFORMATION {
+    Bias: ::LONG,
+    StandardName: [::WCHAR; 32],
+    StandardDate: ::SYSTEMTIME,
+    StandardBias: ::LONG,
+    DaylightName: [::WCHAR; 32],
+    DaylightDate: ::SYSTEMTIME,
+    DaylightBias: ::LONG,
+}}
 pub type PTIME_ZONE_INFORMATION = *mut TIME_ZONE_INFORMATION;
 pub type LPTIME_ZONE_INFORMATION = *mut TIME_ZONE_INFORMATION;
 #[repr(C)] #[derive(Copy)]
